# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `IRoi.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_ServicesF_ice
import omero_Collections_ice

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module Glacier2
_M_Glacier2 = Ice.openModule('Glacier2')

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Included module omero.grid
_M_omero.grid = Ice.openModule('omero.grid')

# Start of module omero
__name__ = 'omero'

# Start of module omero.api
__name__ = 'omero.api'

if 'RoiOptions' not in _M_omero.api.__dict__:
    _M_omero.api.RoiOptions = Ice.createTempClass()
    class RoiOptions(Ice.Object):
        """
        Specifies filters used when querying the ROIs.
        """
        def __init__(self, shapes=None, limit=None, offset=None, userId=None, groupId=None):
            self.shapes = shapes
            self.limit = limit
            self.offset = offset
            self.userId = userId
            self.groupId = groupId

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::api::RoiOptions')

        def ice_id(self, current=None):
            return '::omero::api::RoiOptions'

        def ice_staticId():
            return '::omero::api::RoiOptions'
        ice_staticId = staticmethod(ice_staticId)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.api._t_RoiOptions)

        __repr__ = __str__

    _M_omero.api.RoiOptionsPrx = Ice.createTempClass()
    class RoiOptionsPrx(Ice.ObjectPrx):

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.api.RoiOptionsPrx.ice_checkedCast(proxy, '::omero::api::RoiOptions', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.api.RoiOptionsPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::api::RoiOptions'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.api._t_RoiOptionsPrx = IcePy.defineProxy('::omero::api::RoiOptions', RoiOptionsPrx)

    _M_omero.api._t_RoiOptions = IcePy.declareClass('::omero::api::RoiOptions')

    _M_omero.api._t_RoiOptions = IcePy.defineClass('::omero::api::RoiOptions', RoiOptions, -1, (), False, False, None, (), (
        ('shapes', (), _M_omero.api._t_StringSet, False, 0),
        ('limit', (), _M_omero._t_RInt, False, 0),
        ('offset', (), _M_omero._t_RInt, False, 0),
        ('userId', (), _M_omero._t_RLong, False, 0),
        ('groupId', (), _M_omero._t_RLong, False, 0)
    ))
    RoiOptions._ice_type = _M_omero.api._t_RoiOptions

    _M_omero.api.RoiOptions = RoiOptions
    del RoiOptions

    _M_omero.api.RoiOptionsPrx = RoiOptionsPrx
    del RoiOptionsPrx

if 'RoiResult' not in _M_omero.api.__dict__:
    _M_omero.api.RoiResult = Ice.createTempClass()
    class RoiResult(Ice.Object):
        """
        Returned by most search methods. The RoiOptions is the options object passed
        into a method, possibly modified by the server if some value was out of range.
        The RoiList contains all the Rois which matched the given query.
        The individual shapes of the Rois which matched can be found in the indexes.
        For example, all the shapes on z=1 can by found by:
        ShapeList shapes = byZ.get(1);
        Shapes which are found on all z or t can be found with:
        byZ.get(-1);
        byT.get(-1);
        respectively.
        """
        def __init__(self, opts=None, rois=None, byZ=None, byT=None):
            self.opts = opts
            self.rois = rois
            self.byZ = byZ
            self.byT = byT

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::api::RoiResult')

        def ice_id(self, current=None):
            return '::omero::api::RoiResult'

        def ice_staticId():
            return '::omero::api::RoiResult'
        ice_staticId = staticmethod(ice_staticId)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.api._t_RoiResult)

        __repr__ = __str__

    _M_omero.api.RoiResultPrx = Ice.createTempClass()
    class RoiResultPrx(Ice.ObjectPrx):

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.api.RoiResultPrx.ice_checkedCast(proxy, '::omero::api::RoiResult', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.api.RoiResultPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::api::RoiResult'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.api._t_RoiResultPrx = IcePy.defineProxy('::omero::api::RoiResult', RoiResultPrx)

    _M_omero.api._t_RoiResult = IcePy.declareClass('::omero::api::RoiResult')

    _M_omero.api._t_RoiResult = IcePy.defineClass('::omero::api::RoiResult', RoiResult, -1, (), False, False, None, (), (
        ('opts', (), _M_omero.api._t_RoiOptions, False, 0),
        ('rois', (), _M_omero.api._t_RoiList, False, 0),
        ('byZ', (), _M_omero.api._t_IntShapeListMap, False, 0),
        ('byT', (), _M_omero.api._t_IntShapeListMap, False, 0)
    ))
    RoiResult._ice_type = _M_omero.api._t_RoiResult

    _M_omero.api.RoiResult = RoiResult
    del RoiResult

    _M_omero.api.RoiResultPrx = RoiResultPrx
    del RoiResultPrx

if 'ShapePoints' not in _M_omero.api.__dict__:
    _M_omero.api.ShapePoints = Ice.createTempClass()
    class ShapePoints(Ice.Object):
        """
        Contains a discrete representation of the geometry of
        an omero::model::Shape. The x and y array are of the
        same size with each pair of entries representing a
        single point in the 2D plane.
        """
        def __init__(self, x=None, y=None):
            self.x = x
            self.y = y

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::api::ShapePoints')

        def ice_id(self, current=None):
            return '::omero::api::ShapePoints'

        def ice_staticId():
            return '::omero::api::ShapePoints'
        ice_staticId = staticmethod(ice_staticId)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.api._t_ShapePoints)

        __repr__ = __str__

    _M_omero.api.ShapePointsPrx = Ice.createTempClass()
    class ShapePointsPrx(Ice.ObjectPrx):

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.api.ShapePointsPrx.ice_checkedCast(proxy, '::omero::api::ShapePoints', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.api.ShapePointsPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::api::ShapePoints'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.api._t_ShapePointsPrx = IcePy.defineProxy('::omero::api::ShapePoints', ShapePointsPrx)

    _M_omero.api._t_ShapePoints = IcePy.defineClass('::omero::api::ShapePoints', ShapePoints, -1, (), False, False, None, (), (
        ('x', (), _M_omero.api._t_IntegerArray, False, 0),
        ('y', (), _M_omero.api._t_IntegerArray, False, 0)
    ))
    ShapePoints._ice_type = _M_omero.api._t_ShapePoints

    _M_omero.api.ShapePoints = ShapePoints
    del ShapePoints

    _M_omero.api.ShapePointsPrx = ShapePointsPrx
    del ShapePointsPrx

if 'ShapeStats' not in _M_omero.api.__dict__:
    _M_omero.api.ShapeStats = Ice.createTempClass()
    class ShapeStats(Ice.Object):
        """
        Contains arrays, one entry per channel, of the statistics
        for a given shape. All arrays are the same size, except for
        the channelIds array, which specifies the ids of the logical
        channels which compose this Shape. If the user specified no
        logical channels for the Shape, then all logical channels from
        the Pixels will be in channelIds.
        """
        def __init__(self, shapeId=0, channelIds=None, pointsCount=None, min=None, max=None, sum=None, mean=None, stdDev=None):
            self.shapeId = shapeId
            self.channelIds = channelIds
            self.pointsCount = pointsCount
            self.min = min
            self.max = max
            self.sum = sum
            self.mean = mean
            self.stdDev = stdDev

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::api::ShapeStats')

        def ice_id(self, current=None):
            return '::omero::api::ShapeStats'

        def ice_staticId():
            return '::omero::api::ShapeStats'
        ice_staticId = staticmethod(ice_staticId)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.api._t_ShapeStats)

        __repr__ = __str__

    _M_omero.api.ShapeStatsPrx = Ice.createTempClass()
    class ShapeStatsPrx(Ice.ObjectPrx):

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.api.ShapeStatsPrx.ice_checkedCast(proxy, '::omero::api::ShapeStats', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.api.ShapeStatsPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::api::ShapeStats'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.api._t_ShapeStatsPrx = IcePy.defineProxy('::omero::api::ShapeStats', ShapeStatsPrx)

    _M_omero.api._t_ShapeStats = IcePy.defineClass('::omero::api::ShapeStats', ShapeStats, -1, (), False, False, None, (), (
        ('shapeId', (), IcePy._t_long, False, 0),
        ('channelIds', (), _M_omero.api._t_LongArray, False, 0),
        ('pointsCount', (), _M_omero.api._t_LongArray, False, 0),
        ('min', (), _M_omero.api._t_DoubleArray, False, 0),
        ('max', (), _M_omero.api._t_DoubleArray, False, 0),
        ('sum', (), _M_omero.api._t_DoubleArray, False, 0),
        ('mean', (), _M_omero.api._t_DoubleArray, False, 0),
        ('stdDev', (), _M_omero.api._t_DoubleArray, False, 0)
    ))
    ShapeStats._ice_type = _M_omero.api._t_ShapeStats

    _M_omero.api.ShapeStats = ShapeStats
    del ShapeStats

    _M_omero.api.ShapeStatsPrx = ShapeStatsPrx
    del ShapeStatsPrx

if '_t_ShapeStatsList' not in _M_omero.api.__dict__:
    _M_omero.api._t_ShapeStatsList = IcePy.defineSequence('::omero::api::ShapeStatsList', (), _M_omero.api._t_ShapeStats)

if '_t_LongRoiResultMap' not in _M_omero.api.__dict__:
    _M_omero.api._t_LongRoiResultMap = IcePy.defineDictionary('::omero::api::LongRoiResultMap', (), IcePy._t_long, _M_omero.api._t_RoiResult)

if 'RoiStats' not in _M_omero.api.__dict__:
    _M_omero.api.RoiStats = Ice.createTempClass()
    class RoiStats(Ice.Object):
        """
        Container for ShapeStats, one with the combined values,
        and one per shape.
        """
        def __init__(self, roiId=0, imageId=0, pixelsId=0, combined=None, perShape=None):
            self.roiId = roiId
            self.imageId = imageId
            self.pixelsId = pixelsId
            self.combined = combined
            self.perShape = perShape

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::api::RoiStats')

        def ice_id(self, current=None):
            return '::omero::api::RoiStats'

        def ice_staticId():
            return '::omero::api::RoiStats'
        ice_staticId = staticmethod(ice_staticId)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.api._t_RoiStats)

        __repr__ = __str__

    _M_omero.api.RoiStatsPrx = Ice.createTempClass()
    class RoiStatsPrx(Ice.ObjectPrx):

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.api.RoiStatsPrx.ice_checkedCast(proxy, '::omero::api::RoiStats', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.api.RoiStatsPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::api::RoiStats'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.api._t_RoiStatsPrx = IcePy.defineProxy('::omero::api::RoiStats', RoiStatsPrx)

    _M_omero.api._t_RoiStats = IcePy.declareClass('::omero::api::RoiStats')

    _M_omero.api._t_RoiStats = IcePy.defineClass('::omero::api::RoiStats', RoiStats, -1, (), False, False, None, (), (
        ('roiId', (), IcePy._t_long, False, 0),
        ('imageId', (), IcePy._t_long, False, 0),
        ('pixelsId', (), IcePy._t_long, False, 0),
        ('combined', (), _M_omero.api._t_ShapeStats, False, 0),
        ('perShape', (), _M_omero.api._t_ShapeStatsList, False, 0)
    ))
    RoiStats._ice_type = _M_omero.api._t_RoiStats

    _M_omero.api.RoiStats = RoiStats
    del RoiStats

    _M_omero.api.RoiStatsPrx = RoiStatsPrx
    del RoiStatsPrx

if 'IRoi' not in _M_omero.api.__dict__:
    _M_omero.api.IRoi = Ice.createTempClass()
    class IRoi(_M_omero.api.ServiceInterface):
        """
        Interface for working with regions of interest.
        """
        def __init__(self):
            if Ice.getType(self) == _M_omero.api.IRoi:
                raise RuntimeError('omero.api.IRoi is an abstract class')

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::api::IRoi', '::omero::api::ServiceInterface')

        def ice_id(self, current=None):
            return '::omero::api::IRoi'

        def ice_staticId():
            return '::omero::api::IRoi'
        ice_staticId = staticmethod(ice_staticId)

        def findByRoi_async(self, _cb, roiId, opts, current=None):
            """
            Returns a RoiResult with a single Roi member.
            Shape linkages are properly created.
            All Shapes are loaded, as is the Pixels and Image object.
            TODO: Annotations?
            Arguments:
            _cb -- The asynchronous callback object.
            roiId -- 
            opts -- 
            current -- The Current object for the invocation.
            """
            pass

        def findByImage_async(self, _cb, imageId, opts, current=None):
            """
            Returns all the Rois in an Image, indexed via Shape.
            Loads Rois as findByRoi.
            Arguments:
            _cb -- The asynchronous callback object.
            imageId -- 
            opts -- 
            current -- The Current object for the invocation.
            """
            pass

        def findByPlane_async(self, _cb, imageId, z, t, opts, current=None):
            """
            Returns all the Rois on the given plane, indexed via Shape.
            Loads Rois as findByRoi.
            Arguments:
            _cb -- The asynchronous callback object.
            imageId -- 
            z -- 
            t -- 
            opts -- 
            current -- The Current object for the invocation.
            """
            pass

        def getPoints_async(self, _cb, shapeId, current=None):
            """
            Calculate the points contained within a given shape
            Arguments:
            _cb -- The asynchronous callback object.
            shapeId -- 
            current -- The Current object for the invocation.
            """
            pass

        def getRoiStats_async(self, _cb, roiId, current=None):
            """
            Calculate stats for all the shapes within the given Roi.
            Arguments:
            _cb -- The asynchronous callback object.
            roiId -- 
            current -- The Current object for the invocation.
            """
            pass

        def getShapeStats_async(self, _cb, shapeId, current=None):
            """
            Calculate the stats for the points within the given Shape.
            Arguments:
            _cb -- The asynchronous callback object.
            shapeId -- 
            current -- The Current object for the invocation.
            """
            pass

        def getShapeStatsList_async(self, _cb, shapeIdList, current=None):
            """
            Calculate the stats for the points within the given Shapes.
            Arguments:
            _cb -- The asynchronous callback object.
            shapeIdList -- 
            current -- The Current object for the invocation.
            """
            pass

        def getShapeStatsRestricted_async(self, _cb, shapeIdList, zForUnattached, tForUnattached, channels, current=None):
            """
            Calculate the stats for the points within the given Shapes.
            Varies to the above in the following ways:
            - does not allow tiled images
            - shapes have to be all belonging to the same image
            - unattached z/t use the fallback parameters zForUnattached/tForUnattached
            that is to say there is never more than 1 z/t combination queried
            - if channel list is given, only the channels in that list are iterated over
            - does not request data from reader on each iteration
            Arguments:
            _cb -- The asynchronous callback object.
            shapeIdList -- 
            zForUnattached -- 
            tForUnattached -- 
            channels -- 
            current -- The Current object for the invocation.
            """
            pass

        def getRoiMeasurements_async(self, _cb, imageId, opts, current=None):
            """
            Returns a list of omero.model.FileAnnotation
            instances with the namespace
            openmicroscopy.org/measurements which are attached
            to the omero.model.Plate containing the given image
            AND which are attached to at least one
            omero.model.Roi
            Arguments:
            _cb -- The asynchronous callback object.
            imageId -- 
            opts -- 
            current -- The Current object for the invocation.
            """
            pass

        def getMeasuredRois_async(self, _cb, imageId, annotationId, opts, current=None):
            """
            Loads the ROIs which are linked to by the given
            omero.model.FileAnnotation id for the given image.
            Arguments:
            _cb -- The asynchronous callback object.
            imageId -- 
            annotationId -- if -1, logic is identical to findByImage(imageId, opts)
            opts -- 
            current -- The Current object for the invocation.
            """
            pass

        def getMeasuredRoisMap_async(self, _cb, imageId, annotationIds, opts, current=None):
            """
            Returns a map from omero.model.FileAnnotation ids
            to RoiResult instances.
            Logic is identical to getMeasuredRois, but Roi data will not be duplicated. (i.e.
            the objects are referentially identical)
            Arguments:
            _cb -- The asynchronous callback object.
            imageId -- 
            annotationIds -- 
            opts -- 
            current -- The Current object for the invocation.
            """
            pass

        def getTable_async(self, _cb, annotationId, current=None):
            """
            Returns the OMERO.tables service via the
            omero.model.FileAnnotation id returned
            by {@code getImageMeasurements}.
            Arguments:
            _cb -- The asynchronous callback object.
            annotationId -- 
            current -- The Current object for the invocation.
            """
            pass

        def uploadMask_async(self, _cb, roiId, z, t, bytes, current=None):
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.api._t_IRoi)

        __repr__ = __str__

    _M_omero.api.IRoiPrx = Ice.createTempClass()
    class IRoiPrx(_M_omero.api.ServiceInterfacePrx):

        """
        Returns a RoiResult with a single Roi member.
        Shape linkages are properly created.
        All Shapes are loaded, as is the Pixels and Image object.
        TODO: Annotations?
        Arguments:
        roiId -- 
        opts -- 
        _ctx -- The request context for the invocation.
        """
        def findByRoi(self, roiId, opts, _ctx=None):
            return _M_omero.api.IRoi._op_findByRoi.invoke(self, ((roiId, opts), _ctx))

        """
        Returns a RoiResult with a single Roi member.
        Shape linkages are properly created.
        All Shapes are loaded, as is the Pixels and Image object.
        TODO: Annotations?
        Arguments:
        roiId -- 
        opts -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_findByRoi(self, roiId, opts, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IRoi._op_findByRoi.begin(self, ((roiId, opts), _response, _ex, _sent, _ctx))

        """
        Returns a RoiResult with a single Roi member.
        Shape linkages are properly created.
        All Shapes are loaded, as is the Pixels and Image object.
        TODO: Annotations?
        Arguments:
        roiId -- 
        opts -- 
        """
        def end_findByRoi(self, _r):
            return _M_omero.api.IRoi._op_findByRoi.end(self, _r)

        """
        Returns all the Rois in an Image, indexed via Shape.
        Loads Rois as findByRoi.
        Arguments:
        imageId -- 
        opts -- 
        _ctx -- The request context for the invocation.
        """
        def findByImage(self, imageId, opts, _ctx=None):
            return _M_omero.api.IRoi._op_findByImage.invoke(self, ((imageId, opts), _ctx))

        """
        Returns all the Rois in an Image, indexed via Shape.
        Loads Rois as findByRoi.
        Arguments:
        imageId -- 
        opts -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_findByImage(self, imageId, opts, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IRoi._op_findByImage.begin(self, ((imageId, opts), _response, _ex, _sent, _ctx))

        """
        Returns all the Rois in an Image, indexed via Shape.
        Loads Rois as findByRoi.
        Arguments:
        imageId -- 
        opts -- 
        """
        def end_findByImage(self, _r):
            return _M_omero.api.IRoi._op_findByImage.end(self, _r)

        """
        Returns all the Rois on the given plane, indexed via Shape.
        Loads Rois as findByRoi.
        Arguments:
        imageId -- 
        z -- 
        t -- 
        opts -- 
        _ctx -- The request context for the invocation.
        """
        def findByPlane(self, imageId, z, t, opts, _ctx=None):
            return _M_omero.api.IRoi._op_findByPlane.invoke(self, ((imageId, z, t, opts), _ctx))

        """
        Returns all the Rois on the given plane, indexed via Shape.
        Loads Rois as findByRoi.
        Arguments:
        imageId -- 
        z -- 
        t -- 
        opts -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_findByPlane(self, imageId, z, t, opts, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IRoi._op_findByPlane.begin(self, ((imageId, z, t, opts), _response, _ex, _sent, _ctx))

        """
        Returns all the Rois on the given plane, indexed via Shape.
        Loads Rois as findByRoi.
        Arguments:
        imageId -- 
        z -- 
        t -- 
        opts -- 
        """
        def end_findByPlane(self, _r):
            return _M_omero.api.IRoi._op_findByPlane.end(self, _r)

        """
        Calculate the points contained within a given shape
        Arguments:
        shapeId -- 
        _ctx -- The request context for the invocation.
        """
        def getPoints(self, shapeId, _ctx=None):
            return _M_omero.api.IRoi._op_getPoints.invoke(self, ((shapeId, ), _ctx))

        """
        Calculate the points contained within a given shape
        Arguments:
        shapeId -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getPoints(self, shapeId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IRoi._op_getPoints.begin(self, ((shapeId, ), _response, _ex, _sent, _ctx))

        """
        Calculate the points contained within a given shape
        Arguments:
        shapeId -- 
        """
        def end_getPoints(self, _r):
            return _M_omero.api.IRoi._op_getPoints.end(self, _r)

        """
        Calculate stats for all the shapes within the given Roi.
        Arguments:
        roiId -- 
        _ctx -- The request context for the invocation.
        """
        def getRoiStats(self, roiId, _ctx=None):
            return _M_omero.api.IRoi._op_getRoiStats.invoke(self, ((roiId, ), _ctx))

        """
        Calculate stats for all the shapes within the given Roi.
        Arguments:
        roiId -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getRoiStats(self, roiId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IRoi._op_getRoiStats.begin(self, ((roiId, ), _response, _ex, _sent, _ctx))

        """
        Calculate stats for all the shapes within the given Roi.
        Arguments:
        roiId -- 
        """
        def end_getRoiStats(self, _r):
            return _M_omero.api.IRoi._op_getRoiStats.end(self, _r)

        """
        Calculate the stats for the points within the given Shape.
        Arguments:
        shapeId -- 
        _ctx -- The request context for the invocation.
        """
        def getShapeStats(self, shapeId, _ctx=None):
            return _M_omero.api.IRoi._op_getShapeStats.invoke(self, ((shapeId, ), _ctx))

        """
        Calculate the stats for the points within the given Shape.
        Arguments:
        shapeId -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getShapeStats(self, shapeId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IRoi._op_getShapeStats.begin(self, ((shapeId, ), _response, _ex, _sent, _ctx))

        """
        Calculate the stats for the points within the given Shape.
        Arguments:
        shapeId -- 
        """
        def end_getShapeStats(self, _r):
            return _M_omero.api.IRoi._op_getShapeStats.end(self, _r)

        """
        Calculate the stats for the points within the given Shapes.
        Arguments:
        shapeIdList -- 
        _ctx -- The request context for the invocation.
        """
        def getShapeStatsList(self, shapeIdList, _ctx=None):
            return _M_omero.api.IRoi._op_getShapeStatsList.invoke(self, ((shapeIdList, ), _ctx))

        """
        Calculate the stats for the points within the given Shapes.
        Arguments:
        shapeIdList -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getShapeStatsList(self, shapeIdList, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IRoi._op_getShapeStatsList.begin(self, ((shapeIdList, ), _response, _ex, _sent, _ctx))

        """
        Calculate the stats for the points within the given Shapes.
        Arguments:
        shapeIdList -- 
        """
        def end_getShapeStatsList(self, _r):
            return _M_omero.api.IRoi._op_getShapeStatsList.end(self, _r)

        """
        Calculate the stats for the points within the given Shapes.
        Varies to the above in the following ways:
        - does not allow tiled images
        - shapes have to be all belonging to the same image
        - unattached z/t use the fallback parameters zForUnattached/tForUnattached
        that is to say there is never more than 1 z/t combination queried
        - if channel list is given, only the channels in that list are iterated over
        - does not request data from reader on each iteration
        Arguments:
        shapeIdList -- 
        zForUnattached -- 
        tForUnattached -- 
        channels -- 
        _ctx -- The request context for the invocation.
        """
        def getShapeStatsRestricted(self, shapeIdList, zForUnattached, tForUnattached, channels, _ctx=None):
            return _M_omero.api.IRoi._op_getShapeStatsRestricted.invoke(self, ((shapeIdList, zForUnattached, tForUnattached, channels), _ctx))

        """
        Calculate the stats for the points within the given Shapes.
        Varies to the above in the following ways:
        - does not allow tiled images
        - shapes have to be all belonging to the same image
        - unattached z/t use the fallback parameters zForUnattached/tForUnattached
        that is to say there is never more than 1 z/t combination queried
        - if channel list is given, only the channels in that list are iterated over
        - does not request data from reader on each iteration
        Arguments:
        shapeIdList -- 
        zForUnattached -- 
        tForUnattached -- 
        channels -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getShapeStatsRestricted(self, shapeIdList, zForUnattached, tForUnattached, channels, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IRoi._op_getShapeStatsRestricted.begin(self, ((shapeIdList, zForUnattached, tForUnattached, channels), _response, _ex, _sent, _ctx))

        """
        Calculate the stats for the points within the given Shapes.
        Varies to the above in the following ways:
        - does not allow tiled images
        - shapes have to be all belonging to the same image
        - unattached z/t use the fallback parameters zForUnattached/tForUnattached
        that is to say there is never more than 1 z/t combination queried
        - if channel list is given, only the channels in that list are iterated over
        - does not request data from reader on each iteration
        Arguments:
        shapeIdList -- 
        zForUnattached -- 
        tForUnattached -- 
        channels -- 
        """
        def end_getShapeStatsRestricted(self, _r):
            return _M_omero.api.IRoi._op_getShapeStatsRestricted.end(self, _r)

        """
        Returns a list of omero.model.FileAnnotation
        instances with the namespace
        openmicroscopy.org/measurements which are attached
        to the omero.model.Plate containing the given image
        AND which are attached to at least one
        omero.model.Roi
        Arguments:
        imageId -- 
        opts -- 
        _ctx -- The request context for the invocation.
        """
        def getRoiMeasurements(self, imageId, opts, _ctx=None):
            return _M_omero.api.IRoi._op_getRoiMeasurements.invoke(self, ((imageId, opts), _ctx))

        """
        Returns a list of omero.model.FileAnnotation
        instances with the namespace
        openmicroscopy.org/measurements which are attached
        to the omero.model.Plate containing the given image
        AND which are attached to at least one
        omero.model.Roi
        Arguments:
        imageId -- 
        opts -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getRoiMeasurements(self, imageId, opts, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IRoi._op_getRoiMeasurements.begin(self, ((imageId, opts), _response, _ex, _sent, _ctx))

        """
        Returns a list of omero.model.FileAnnotation
        instances with the namespace
        openmicroscopy.org/measurements which are attached
        to the omero.model.Plate containing the given image
        AND which are attached to at least one
        omero.model.Roi
        Arguments:
        imageId -- 
        opts -- 
        """
        def end_getRoiMeasurements(self, _r):
            return _M_omero.api.IRoi._op_getRoiMeasurements.end(self, _r)

        """
        Loads the ROIs which are linked to by the given
        omero.model.FileAnnotation id for the given image.
        Arguments:
        imageId -- 
        annotationId -- if -1, logic is identical to findByImage(imageId, opts)
        opts -- 
        _ctx -- The request context for the invocation.
        """
        def getMeasuredRois(self, imageId, annotationId, opts, _ctx=None):
            return _M_omero.api.IRoi._op_getMeasuredRois.invoke(self, ((imageId, annotationId, opts), _ctx))

        """
        Loads the ROIs which are linked to by the given
        omero.model.FileAnnotation id for the given image.
        Arguments:
        imageId -- 
        annotationId -- if -1, logic is identical to findByImage(imageId, opts)
        opts -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getMeasuredRois(self, imageId, annotationId, opts, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IRoi._op_getMeasuredRois.begin(self, ((imageId, annotationId, opts), _response, _ex, _sent, _ctx))

        """
        Loads the ROIs which are linked to by the given
        omero.model.FileAnnotation id for the given image.
        Arguments:
        imageId -- 
        annotationId -- if -1, logic is identical to findByImage(imageId, opts)
        opts -- 
        """
        def end_getMeasuredRois(self, _r):
            return _M_omero.api.IRoi._op_getMeasuredRois.end(self, _r)

        """
        Returns a map from omero.model.FileAnnotation ids
        to RoiResult instances.
        Logic is identical to getMeasuredRois, but Roi data will not be duplicated. (i.e.
        the objects are referentially identical)
        Arguments:
        imageId -- 
        annotationIds -- 
        opts -- 
        _ctx -- The request context for the invocation.
        """
        def getMeasuredRoisMap(self, imageId, annotationIds, opts, _ctx=None):
            return _M_omero.api.IRoi._op_getMeasuredRoisMap.invoke(self, ((imageId, annotationIds, opts), _ctx))

        """
        Returns a map from omero.model.FileAnnotation ids
        to RoiResult instances.
        Logic is identical to getMeasuredRois, but Roi data will not be duplicated. (i.e.
        the objects are referentially identical)
        Arguments:
        imageId -- 
        annotationIds -- 
        opts -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getMeasuredRoisMap(self, imageId, annotationIds, opts, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IRoi._op_getMeasuredRoisMap.begin(self, ((imageId, annotationIds, opts), _response, _ex, _sent, _ctx))

        """
        Returns a map from omero.model.FileAnnotation ids
        to RoiResult instances.
        Logic is identical to getMeasuredRois, but Roi data will not be duplicated. (i.e.
        the objects are referentially identical)
        Arguments:
        imageId -- 
        annotationIds -- 
        opts -- 
        """
        def end_getMeasuredRoisMap(self, _r):
            return _M_omero.api.IRoi._op_getMeasuredRoisMap.end(self, _r)

        """
        Returns the OMERO.tables service via the
        omero.model.FileAnnotation id returned
        by {@code getImageMeasurements}.
        Arguments:
        annotationId -- 
        _ctx -- The request context for the invocation.
        """
        def getTable(self, annotationId, _ctx=None):
            return _M_omero.api.IRoi._op_getTable.invoke(self, ((annotationId, ), _ctx))

        """
        Returns the OMERO.tables service via the
        omero.model.FileAnnotation id returned
        by {@code getImageMeasurements}.
        Arguments:
        annotationId -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getTable(self, annotationId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IRoi._op_getTable.begin(self, ((annotationId, ), _response, _ex, _sent, _ctx))

        """
        Returns the OMERO.tables service via the
        omero.model.FileAnnotation id returned
        by {@code getImageMeasurements}.
        Arguments:
        annotationId -- 
        """
        def end_getTable(self, _r):
            return _M_omero.api.IRoi._op_getTable.end(self, _r)

        def uploadMask(self, roiId, z, t, bytes, _ctx=None):
            return _M_omero.api.IRoi._op_uploadMask.invoke(self, ((roiId, z, t, bytes), _ctx))

        def begin_uploadMask(self, roiId, z, t, bytes, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IRoi._op_uploadMask.begin(self, ((roiId, z, t, bytes), _response, _ex, _sent, _ctx))

        def end_uploadMask(self, _r):
            return _M_omero.api.IRoi._op_uploadMask.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.api.IRoiPrx.ice_checkedCast(proxy, '::omero::api::IRoi', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.api.IRoiPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::api::IRoi'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.api._t_IRoiPrx = IcePy.defineProxy('::omero::api::IRoi', IRoiPrx)

    _M_omero.api._t_IRoi = IcePy.defineClass('::omero::api::IRoi', IRoi, -1, (), True, False, None, (_M_omero.api._t_ServiceInterface,), ())
    IRoi._ice_type = _M_omero.api._t_IRoi

    IRoi._op_findByRoi = IcePy.Operation('findByRoi', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_long, False, 0), ((), _M_omero.api._t_RoiOptions, False, 0)), (), ((), _M_omero.api._t_RoiResult, False, 0), (_M_omero._t_ServerError,))
    IRoi._op_findByRoi.deprecate("IROI is deprecated.")
    IRoi._op_findByImage = IcePy.Operation('findByImage', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_long, False, 0), ((), _M_omero.api._t_RoiOptions, False, 0)), (), ((), _M_omero.api._t_RoiResult, False, 0), (_M_omero._t_ServerError,))
    IRoi._op_findByImage.deprecate("IROI is deprecated.")
    IRoi._op_findByPlane = IcePy.Operation('findByPlane', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_long, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), _M_omero.api._t_RoiOptions, False, 0)), (), ((), _M_omero.api._t_RoiResult, False, 0), (_M_omero._t_ServerError,))
    IRoi._op_findByPlane.deprecate("IROI is deprecated.")
    IRoi._op_getPoints = IcePy.Operation('getPoints', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_long, False, 0),), (), ((), _M_omero.api._t_ShapePoints, False, 0), (_M_omero._t_ServerError,))
    IRoi._op_getPoints.deprecate("IROI is deprecated.")
    IRoi._op_getRoiStats = IcePy.Operation('getRoiStats', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_long, False, 0),), (), ((), _M_omero.api._t_RoiStats, False, 0), (_M_omero._t_ServerError,))
    IRoi._op_getRoiStats.deprecate("IROI is deprecated.")
    IRoi._op_getShapeStats = IcePy.Operation('getShapeStats', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_long, False, 0),), (), ((), _M_omero.api._t_ShapeStats, False, 0), (_M_omero._t_ServerError,))
    IRoi._op_getShapeStats.deprecate("IROI is deprecated.")
    IRoi._op_getShapeStatsList = IcePy.Operation('getShapeStatsList', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), _M_omero.api._t_LongList, False, 0),), (), ((), _M_omero.api._t_ShapeStatsList, False, 0), (_M_omero._t_ServerError,))
    IRoi._op_getShapeStatsList.deprecate("IROI is deprecated.")
    IRoi._op_getShapeStatsRestricted = IcePy.Operation('getShapeStatsRestricted', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), _M_omero.api._t_LongList, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), _M_omero.api._t_IntegerArray, False, 0)), (), ((), _M_omero.api._t_ShapeStatsList, False, 0), (_M_omero._t_ServerError,))
    IRoi._op_getShapeStatsRestricted.deprecate("IROI is deprecated.")
    IRoi._op_getRoiMeasurements = IcePy.Operation('getRoiMeasurements', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_long, False, 0), ((), _M_omero.api._t_RoiOptions, False, 0)), (), ((), _M_omero.api._t_AnnotationList, False, 0), (_M_omero._t_ServerError,))
    IRoi._op_getRoiMeasurements.deprecate("IROI is deprecated.")
    IRoi._op_getMeasuredRois = IcePy.Operation('getMeasuredRois', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_long, False, 0), ((), IcePy._t_long, False, 0), ((), _M_omero.api._t_RoiOptions, False, 0)), (), ((), _M_omero.api._t_RoiResult, False, 0), (_M_omero._t_ServerError,))
    IRoi._op_getMeasuredRois.deprecate("IROI is deprecated.")
    IRoi._op_getMeasuredRoisMap = IcePy.Operation('getMeasuredRoisMap', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_long, False, 0), ((), _M_omero.api._t_LongList, False, 0), ((), _M_omero.api._t_RoiOptions, False, 0)), (), ((), _M_omero.api._t_LongRoiResultMap, False, 0), (_M_omero._t_ServerError,))
    IRoi._op_getMeasuredRoisMap.deprecate("IROI is deprecated.")
    IRoi._op_getTable = IcePy.Operation('getTable', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_long, False, 0),), (), ((), _M_omero.grid._t_TablePrx, False, 0), (_M_omero._t_ServerError,))
    IRoi._op_getTable.deprecate("IROI is deprecated.")
    IRoi._op_uploadMask = IcePy.Operation('uploadMask', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), IcePy._t_long, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), _M_Ice._t_ByteSeq, False, 0)), (), None, (_M_omero._t_ServerError,))
    IRoi._op_uploadMask.deprecate("IROI is deprecated.")

    _M_omero.api.IRoi = IRoi
    del IRoi

    _M_omero.api.IRoiPrx = IRoiPrx
    del IRoiPrx

# End of module omero.api

__name__ = 'omero'

# End of module omero



"""
   /*
   **   Generated by blitz/resources/templates/combined.vm
   **
   **   Copyright 2007, 2008 Glencoe Software, Inc. All rights reserved.
   **   Use is subject to license terms supplied in LICENSE.txt
   **
   */
"""
try:
   unicode
except NameError:
   # Python 3: "unicode" is built-in
   unicode = str
import Ice
import IceImport
import omero
IceImport.load("omero_model_DetailsI")
IceImport.load("omero_model_Channel_ice")
from omero.rtypes import rlong
from collections import namedtuple
_omero = Ice.openModule("omero")
_omero_model = Ice.openModule("omero.model")
__name__ = "omero.model"
class ChannelI(_omero_model.Channel):

      # Property Metadata
      _field_info_data = namedtuple("FieldData", ["wrapper", "nullable"])
      _field_info_type = namedtuple("FieldInfo", [
          "statsInfo",
          "red",
          "green",
          "blue",
          "alpha",
          "lookupTable",
          "logicalChannel",
          "pixels",
          "annotationLinks",
          "details",
      ])
      _field_info = _field_info_type(
          statsInfo=_field_info_data(wrapper=omero.proxy_to_instance, nullable=True),
          red=_field_info_data(wrapper=omero.rtypes.rint, nullable=True),
          green=_field_info_data(wrapper=omero.rtypes.rint, nullable=True),
          blue=_field_info_data(wrapper=omero.rtypes.rint, nullable=True),
          alpha=_field_info_data(wrapper=omero.rtypes.rint, nullable=True),
          lookupTable=_field_info_data(wrapper=omero.rtypes.rstring, nullable=True),
          logicalChannel=_field_info_data(wrapper=omero.proxy_to_instance, nullable=False),
          pixels=_field_info_data(wrapper=omero.proxy_to_instance, nullable=False),
          annotationLinks=_field_info_data(wrapper=omero.proxy_to_instance, nullable=True),
          details=_field_info_data(wrapper=omero.proxy_to_instance, nullable=True),
      )  # end _field_info
      STATSINFO =  "ome.model.core.Channel_statsInfo"
      RED =  "ome.model.core.Channel_red"
      GREEN =  "ome.model.core.Channel_green"
      BLUE =  "ome.model.core.Channel_blue"
      ALPHA =  "ome.model.core.Channel_alpha"
      LOOKUPTABLE =  "ome.model.core.Channel_lookupTable"
      LOGICALCHANNEL =  "ome.model.core.Channel_logicalChannel"
      PIXELS =  "ome.model.core.Channel_pixels"
      ANNOTATIONLINKS =  "ome.model.core.Channel_annotationLinks"
      DETAILS =  "ome.model.core.Channel_details"
      def errorIfUnloaded(self):
          if not self._loaded:
              raise _omero.UnloadedEntityException("Object unloaded:"+str(self))

      def throwNullCollectionException(self,propertyName):
          raise _omero.UnloadedEntityException(""+
          "Error updating collection:" + propertyName +"\n"+
          "Collection is currently null. This can be seen\n" +
          "by testing \""+ propertyName +"Loaded\". This implies\n"+
          "that this collection was unloaded. Please refresh this object\n"+
          "in order to update this collection.\n")

      def _toggleCollectionsLoaded(self, load):
          if load:
              self._annotationLinksSeq = []
              self._annotationLinksLoaded = True;
          else:
              self._annotationLinksSeq = []
              self._annotationLinksLoaded = False;

          pass

      def __init__(self, id=None, loaded=None):
          super(ChannelI, self).__init__()
          if id is not None and isinstance(id, (str, unicode)) and ":" in id:
              parts = id.split(":")
              if len(parts) != 2:
                  raise Exception("Invalid proxy string: %s", id)
              if parts[0] != self.__class__.__name__ and \
                 parts[0]+"I" != self.__class__.__name__:
                  raise Exception("Proxy class mismatch: %s<>%s" %
                  (self.__class__.__name__, parts[0]))
              self._id = rlong(parts[1])
              if loaded is None:
                  # If no loadedness was requested with
                  # a proxy string, then assume False.
                  loaded = False
          else:
              # Relying on omero.rtypes.rlong's error-handling
              self._id = rlong(id)
              if loaded is None:
                  loaded = True  # Assume true as previously
          self._loaded = loaded
          if self._loaded:
             self._details = _omero_model.DetailsI()
             self._toggleCollectionsLoaded(True)

      def unload(self, current = None):
          self._loaded = False
          self.unloadStatsInfo( )
          self.unloadRed( )
          self.unloadGreen( )
          self.unloadBlue( )
          self.unloadAlpha( )
          self.unloadLookupTable( )
          self.unloadLogicalChannel( )
          self.unloadPixels( )
          self.unloadAnnotationLinks( )
          self.unloadDetails( )

      def isLoaded(self, current = None):
          return self._loaded
      def unloadCollections(self, current = None):
          self._toggleCollectionsLoaded( False )
      def isGlobal(self, current = None):
          return  False ;
      def isMutable(self, current = None):
          return  True ;
      def isAnnotated(self, current = None):
          return  True ;
      def isLink(self, current = None):
          return  False ;
      def shallowCopy(self, current = None):
            if not self._loaded: return self.proxy()
            copy = ChannelI()
            copy._id = self._id;
            copy._version = self._version;
            copy._details = None  # Unloading for the moment.
            raise omero.ClientError("NYI")
      def proxy(self, current = None):
          if self._id is None: raise omero.ClientError("Proxies require an id")
          return ChannelI( self._id.getValue(), False )

      def getDetails(self, current = None):
          self.errorIfUnloaded()
          return self._details

      def unloadDetails(self, current = None):
          self._details = None

      def getId(self, current = None):
          return self._id

      def setId(self, _id, current = None):
          self._id = _id

      def checkUnloadedProperty(self, value, loadedField):
          if value == None:
              self.__dict__[loadedField] = False
          else:
              self.__dict__[loadedField] = True

      def getVersion(self, current = None):
          self.errorIfUnloaded()
          return self._version

      def setVersion(self, version, current = None):
          self.errorIfUnloaded()
          self._version = version

      def unloadStatsInfo(self, ):
          self._statsInfoLoaded = False
          self._statsInfo = None;

      def getStatsInfo(self, current = None):
          self.errorIfUnloaded()
          return self._statsInfo

      def setStatsInfo(self, _statsInfo, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.statsInfo.wrapper is not None:
              if _statsInfo is not None:
                  _statsInfo = self._field_info.statsInfo.wrapper(_statsInfo)
          self._statsInfo = _statsInfo
          pass

      def unloadRed(self, ):
          self._redLoaded = False
          self._red = None;

      def getRed(self, current = None):
          self.errorIfUnloaded()
          return self._red

      def setRed(self, _red, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.red.wrapper is not None:
              if _red is not None:
                  _red = self._field_info.red.wrapper(_red)
          self._red = _red
          pass

      def unloadGreen(self, ):
          self._greenLoaded = False
          self._green = None;

      def getGreen(self, current = None):
          self.errorIfUnloaded()
          return self._green

      def setGreen(self, _green, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.green.wrapper is not None:
              if _green is not None:
                  _green = self._field_info.green.wrapper(_green)
          self._green = _green
          pass

      def unloadBlue(self, ):
          self._blueLoaded = False
          self._blue = None;

      def getBlue(self, current = None):
          self.errorIfUnloaded()
          return self._blue

      def setBlue(self, _blue, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.blue.wrapper is not None:
              if _blue is not None:
                  _blue = self._field_info.blue.wrapper(_blue)
          self._blue = _blue
          pass

      def unloadAlpha(self, ):
          self._alphaLoaded = False
          self._alpha = None;

      def getAlpha(self, current = None):
          self.errorIfUnloaded()
          return self._alpha

      def setAlpha(self, _alpha, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.alpha.wrapper is not None:
              if _alpha is not None:
                  _alpha = self._field_info.alpha.wrapper(_alpha)
          self._alpha = _alpha
          pass

      def unloadLookupTable(self, ):
          self._lookupTableLoaded = False
          self._lookupTable = None;

      def getLookupTable(self, current = None):
          self.errorIfUnloaded()
          return self._lookupTable

      def setLookupTable(self, _lookupTable, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.lookupTable.wrapper is not None:
              if _lookupTable is not None:
                  _lookupTable = self._field_info.lookupTable.wrapper(_lookupTable)
          self._lookupTable = _lookupTable
          pass

      def unloadLogicalChannel(self, ):
          self._logicalChannelLoaded = False
          self._logicalChannel = None;

      def getLogicalChannel(self, current = None):
          self.errorIfUnloaded()
          return self._logicalChannel

      def setLogicalChannel(self, _logicalChannel, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.logicalChannel.wrapper is not None:
              if _logicalChannel is not None:
                  _logicalChannel = self._field_info.logicalChannel.wrapper(_logicalChannel)
          self._logicalChannel = _logicalChannel
          pass

      def unloadPixels(self, ):
          self._pixelsLoaded = False
          self._pixels = None;

      def getPixels(self, current = None):
          self.errorIfUnloaded()
          return self._pixels

      def setPixels(self, _pixels, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.pixels.wrapper is not None:
              if _pixels is not None:
                  _pixels = self._field_info.pixels.wrapper(_pixels)
          self._pixels = _pixels
          pass

      def unloadAnnotationLinks(self, current = None):
          self._annotationLinksLoaded = False
          self._annotationLinksSeq = None;

      def _getAnnotationLinks(self, current = None):
          self.errorIfUnloaded()
          return self._annotationLinksSeq

      def _setAnnotationLinks(self, _annotationLinks, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.annotationLinksSeq.wrapper is not None:
              if _annotationLinks is not None:
                  _annotationLinks = self._field_info.annotationLinksSeq.wrapper(_annotationLinks)
          self._annotationLinksSeq = _annotationLinks
          self.checkUnloadedProperty(_annotationLinks,'annotationLinksLoaded')

      def isAnnotationLinksLoaded(self):
          return self._annotationLinksLoaded

      def sizeOfAnnotationLinks(self, current = None):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: return -1
          return len(self._annotationLinksSeq)

      def copyAnnotationLinks(self, current = None):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          return list(self._annotationLinksSeq)

      def iterateAnnotationLinks(self):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          return iter(self._annotationLinksSeq)

      def addChannelAnnotationLink(self, target, current = None):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          self._annotationLinksSeq.append( target );
          target.setParent( self )

      def addAllChannelAnnotationLinkSet(self, targets, current = None):
          self.errorIfUnloaded()
          if  not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          self._annotationLinksSeq.extend( targets )
          for target in targets:
              target.setParent( self )

      def removeChannelAnnotationLink(self, target, current = None):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          self._annotationLinksSeq.remove( target )
          target.setParent( None )

      def removeAllChannelAnnotationLinkSet(self, targets, current = None):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          for elt in targets:
              elt.setParent( None )
              self._annotationLinksSeq.remove( elt )

      def clearAnnotationLinks(self, current = None):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          for elt in self._annotationLinksSeq:
              elt.setParent( None )
          self._annotationLinksSeq = list()

      def reloadAnnotationLinks(self, toCopy, current = None):
          self.errorIfUnloaded()
          if self._annotationLinksLoaded:
              raise omero.ClientError("Cannot reload active collection: annotationLinksSeq")
          if not toCopy:
              raise omero.ClientError("Argument cannot be null")
          if toCopy.getId().getValue() != self.getId().getValue():
             raise omero.ClientError("Argument must have the same id as this instance")
          if toCopy.getDetails().getUpdateEvent().getId().getValue() < self.getDetails().getUpdateEvent().getId().getValue():
             raise omero.ClientError("Argument may not be older than this instance")
          copy = toCopy.copyAnnotationLinks() # May also throw
          for elt in copy:
              elt.setParent( self )
          self._annotationLinksSeq = copy
          toCopy.unloadAnnotationLinks()
          self._annotationLinksLoaded = True

      def getAnnotationLinksCountPerOwner(self, current = None):
          return self._annotationLinksCountPerOwner

      def linkAnnotation(self, addition, current = None):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          link = _omero_model.ChannelAnnotationLinkI()
          link.link( self, addition );
          self.addChannelAnnotationLinkToBoth( link, True )
          return link

      def addChannelAnnotationLinkToBoth(self, link, bothSides):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          self._annotationLinksSeq.append( link )

      def findChannelAnnotationLink(self, removal, current = None):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          result = list()
          for link in self._annotationLinksSeq:
              if link.getChild() == removal: result.append(link)
          return result

      def unlinkAnnotation(self, removal, current = None):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          toRemove = self.findChannelAnnotationLink(removal)
          for next in toRemove:
              self.removeChannelAnnotationLinkFromBoth( next, True )

      def removeChannelAnnotationLinkFromBoth(self, link, bothSides, current = None):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          self._annotationLinksSeq.remove( link )

      def linkedAnnotationList(self, current = None):
          self.errorIfUnloaded()
          if not self.annotationLinksLoaded: self.throwNullCollectionException("AnnotationLinks")
          linked = []
          for link in self._annotationLinksSeq:
              linked.append( link.getChild() )
          return linked


      def ice_postUnmarshal(self):
          """
          Provides additional initialization once all data loaded
          """
          pass # Currently unused


      def ice_preMarshal(self):
          """
          Provides additional validation before data is sent
          """
          pass # Currently unused

      def __getattr__(self, name):
          """
          Reroutes all access to object.field through object.getField() or object.isField()
          """
          if "_" in name:  # Ice disallows underscores, so these should be treated normally.
              return object.__getattribute__(self, name)
          field  = "_" + name
          capitalized = name[0].capitalize() + name[1:]
          getter = "get" + capitalized
          questn = "is" + capitalized
          try:
              self.__dict__[field]
              if hasattr(self, getter):
                  method = getattr(self, getter)
                  return method()
              elif hasattr(self, questn):
                  method = getattr(self, questn)
                  return method()
          except:
              pass
          raise AttributeError("'%s' object has no attribute '%s' or '%s'" % (self.__class__.__name__, getter, questn))

      def __setattr__(self, name, value):
          """
          Reroutes all access to object.field through object.getField(), with the caveat
          that all sets on variables starting with "_" are permitted directly.
          """
          if name.startswith("_"):
              self.__dict__[name] = value
              return
          else:
              field  = "_" + name
              setter = "set" + name[0].capitalize() + name[1:]
              if hasattr(self, field) and hasattr(self, setter):
                  method = getattr(self, setter)
                  return method(value)
          raise AttributeError("'%s' object has no attribute '%s'" % (self.__class__.__name__, setter))

_omero_model.ChannelI = ChannelI



# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `IContainer.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_ModelF_ice
import omero_ServicesF_ice
import omero_System_ice
import omero_Collections_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module Glacier2
_M_Glacier2 = Ice.openModule('Glacier2')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Included module omero.grid
_M_omero.grid = Ice.openModule('omero.grid')

# Start of module omero
__name__ = 'omero'

# Start of module omero.api
__name__ = 'omero.api'

if 'IContainer' not in _M_omero.api.__dict__:
    _M_omero.api.IContainer = Ice.createTempClass()
    class IContainer(_M_omero.api.ServiceInterface):
        """
        Provides methods for dealing with the core Pojos of OME.
        Included are:
        Projects, Datasets, Images.
        Read API
        The names of the methods correlate to how the function operates:
        load: start at container objects and work down toward the
        leaves, returning hierarchy (Project-&gt;Dataset-&gt;Image
        find: start at leaf objects and work up to containers,
        returning hierarchy
        get: retrieves only leaves in the hierarchy (currently
        only Images)
        Options Mechanism
        The options are used to add some constraints to the generic method
        e.g. load hierarchy trees images for a given user. This
        mechanism should give us enough flexibility to extend the API if
        necessary, e.g. in some cases we might want to retrieve the images
        with or without annotations
        Most methods take such an options map which is built
        on the client-side using the omero.sys.Parameters class. The
        currently supported options are:
        annotator(Integer): If key exists but value null,
        annotations are retrieved for all objects in the hierarchy where
        they exist; if a valid experimenterID, annotations are only
        retrieved for that user. May not be used
        be all methods. Default: all annotations
        leaves(Boolean): if FALSE omits images from the returned
        hierarchy. May not be used by all methods. Default: true
        experimenter(Integer): enables filtering on a
        per-experimenter basis. This option has a method-specific (and
        possibly context-specific) meaning. Please see the individual
        methods.
        group(Integer): enables filtering on a per-group basis.
        The experimenter value is ignored if present and instead a
        similar filtering is done using all experimenters in the
        given group.
        Write API
        As outlined in TODO, the semantics of the Omero write API are based
        on three rules:
        IObject-valued fields for which isLoaded() returns
        false are assumed filtered
        Collection-valued fields that are null are assumed filtered
        Collection-valued fields for which
        getDetails().isFiltered(String collectionName) returns
        true are assumed filtered. TODO: should we accept isFiltered for
        all fields?
        In each of these cases, the server will reload that given field
        before attempting to save the graph.
        For all write calls, the options map (see below) must contain the
        userId and the userGroupId for the newly created objects. TODO
        umask.
        """
        def __init__(self):
            if Ice.getType(self) == _M_omero.api.IContainer:
                raise RuntimeError('omero.api.IContainer is an abstract class')

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::api::IContainer', '::omero::api::ServiceInterface')

        def ice_id(self, current=None):
            return '::omero::api::IContainer'

        def ice_staticId():
            return '::omero::api::IContainer'
        ice_staticId = staticmethod(ice_staticId)

        def loadContainerHierarchy_async(self, _cb, rootType, rootIds, options, current=None):
            """
            Retrieves hierarchy trees rooted by a given node (unless
            orphan is specified -- See below)
            This method also retrieves the Experimenters linked to the
            objects in the tree. Similarly, all Images will be linked
            to their Pixel objects if included.
            Note that objects are never duplicated. For example, if an
            Experimenter owns all the objects in the retrieved tree,
            then those objects will be linked to the same
            instance of omero.model.Experimenter. Or if an
            Image is contained in more than one Dataset in the
            retrieved tree, then all enclosing
            omero.model.Dataset objects will point
            to the same omero.model.Image object. And
            so on.
            Arguments:
            _cb -- The asynchronous callback object.
            rootType -- The type of the root node. Can be omero.model.Project, omero.model.Dataset, omero.model.Screen or omero.model.Plate. Cannot be null.
            rootIds -- The ids of the root nodes. Can be null if an Experimenter is specified in options, otherwise an Exception is thrown to prevent all images in the entire database from being downloaded.
            options -- Parameters as above. annotator, leaves, orphan, acquisition data used. acquisition data is only relevant for images and taken into account if the images are loaded. If rootNodeIds==null, experimenter|group must be set and filtering will be applied at the Class-level; e.g. to retrieve a user's Projects, or user's Datasets. If rootNodeIds!=null, the result will be filtered by the experimenter|group at the Image and intermediate levels if available. Due to the amount of data potentially linked a Screen/Plate, the leaves option is not taken into account when the root node is a omero.model.Screen. orphan implies that objects which are not contained in an object of rootNodeType should also be returned.
            current -- The Current object for the invocation.
            """
            pass

        def findContainerHierarchies_async(self, _cb, rootType, imageIds, options, current=None):
            """
            Retrieves hierarchy trees in various hierarchies that
            contain the specified Images.
            This method will look for all the containers containing the
            specified Images and then for all containers containing
            those containers and on up the container hierarchy.
            This method returns a Set with all root nodes
            that were
            found. Every root node is linked to the found objects and
            so on until the leaf nodes, which are
            omero.model.Image objects. Note that the type of any
            root node in the returned set can be the given
            rootNodeType, any of its containees or an
            omero.model.Image.
            For example, say that you pass in the ids of six Images:
            i1, i2, i3, i4, i5, i6.
            If the P/D/I hierarchy in the DB looks like this:
            |                  __p1__                     
            |                 /      \                    
            |               _d1_    _d2_      d3          
            |              /    \  /    \     |           
            |             i1     i2     i3    i4    i5  i6
            Then the returned set will contain
            p1, d3, i5, i6. All objects will be properly
            linked up.
            Finally, this method will only retrieve the nodes
            that are connected in a tree to the specified leaf image
            nodes. Back to the previous example, if d1
            contained image img500, then the returned
            object would not contain img500. In a
            similar way, if p1 contained
            ds300 and this dataset weren't linked to any of
            the i1, i2, i3, i4, i5, i6  images, then
            ds300 would not be part of the returned
            tree rooted by p1.
            Arguments:
            _cb -- The asynchronous callback object.
            rootType -- top-most type which will be searched for Can be omero.model.Project. Not null.
            imageIds -- Contains the ids of the Images that sit at the bottom of the trees. Not null.
            options -- Parameters as above. annotator used. experimenter|group may be applied at the top-level only or at each level in the hierarchy, but will not apply to the leaf (Image) level.
            current -- The Current object for the invocation.
            """
            pass

        def getImages_async(self, _cb, rootType, rootIds, options, current=None):
            """
            Retrieve a user's (or all users') images within any given
            container. For example, all images in project, applying
            temporal filtering or pagination.
            Arguments:
            _cb -- The asynchronous callback object.
            rootType -- A Class which will have its hierarchy searched for Images. Not null.
            rootIds -- A set of ids of type rootNodeType Not null.
            options -- Parameters as above. No notion of leaves. experimenter|group apply at the Image level. OPTIONS: - startTime and/or endTime should be Timestamp.valueOf("YYYY-MM-DD hh:mm:ss.ms"); limit and offset are applied at the Image-level. That is, calling with Dataset.class, limit == 10 and offset == 0 will first perform one query to get an effective set of rootNodeIds, then getImages will be called with an effective rootNodeType of Image.class and the new ids. acquisition data is only relevant for images.
            current -- The Current object for the invocation.
            """
            pass

        def getUserImages_async(self, _cb, options, current=None):
            """
            Retrieves a user's images.
            Arguments:
            _cb -- The asynchronous callback object.
            options -- Parameters as above. No notion of leaves. experimenter|group apply at the Image level and must be present.
            current -- The Current object for the invocation.
            """
            pass

        def getImagesByOptions_async(self, _cb, options, current=None):
            """
            Retrieves images by options.
            Arguments:
            _cb -- The asynchronous callback object.
            options -- Parameters as above. No notion of leaves. experimenter|group apply at the Image level and must be present. OPTIONS: - startTime and/or endTime should be Timestamp.valueOf("YYYY-MM-DD hh:mm:ss.ms"). acquisition data is only relevant for images.
            current -- The Current object for the invocation.
            """
            pass

        def getImagesBySplitFilesets_async(self, _cb, included, options, current=None):
            """
            Given a list of IDs of certain entity types, calculates
            which filesets are split such that a non-empty proper
            subset of their images are referenced, directly or
            indirectly, as being included. The return value lists both
            the fileset IDs and the image IDs in ascending order,
            the image ID lists separated by if they were included.
            Warning: following discussion in trac ticket 11019 the
            return type may be changed.
            Arguments:
            _cb -- The asynchronous callback object.
            included -- the entities included
            options -- parameters, presently ignored
            current -- The Current object for the invocation.
            """
            pass

        def getCollectionCount_async(self, _cb, type, property, ids, options, current=None):
            """
            Counts the number of members in a collection for a given
            object. For example, if you wanted to retrieve the number
            of Images contained in a Dataset you would pass TODO.
            Arguments:
            _cb -- The asynchronous callback object.
            type -- The fully-qualified classname of the object to be tested
            property -- Name of the property on that class, omitting getters and setters.
            ids -- Set of Longs, the ids of the objects to test
            options -- Parameters. Unused.
            current -- The Current object for the invocation.
            """
            pass

        def retrieveCollection_async(self, _cb, obj, collectionName, options, current=None):
            """
            Retrieves a collection with all members initialized
            (loaded). This is useful when a collection has been
            nulled in a previous query.
            Arguments:
            _cb -- The asynchronous callback object.
            obj -- Can be unloaded.
            collectionName -- public static final String from the IObject.class
            options -- Parameters. Unused.
            current -- The Current object for the invocation.
            """
            pass

        def createDataObject_async(self, _cb, obj, options, current=None):
            """
            Creates the specified data object.
            A placeholder parent object is created if the data object
            is to be put in a collection.
            For example, if the object is a Dataset, we
            first create a Project as parent then we set
            the Dataset parent as follows: 
            //pseudo-code TODO
            Project p = new Project(id,false);
            dataset.addProject(p);
            then for each parent relationship a DataObject
            ome.model.ILink is created.
            Arguments:
            _cb -- The asynchronous callback object.
            obj -- IObject. Supported: Project, Dataset, Annotation, Group, Experimenter. Not null.
            options -- Parameters as above.
            current -- The Current object for the invocation.
            """
            pass

        def createDataObjects_async(self, _cb, dataObjects, options, current=None):
            """
            Convenience method to save network calls. Loops over the
            array of IObjects calling {@code createDataObject}.
            Arguments:
            _cb -- The asynchronous callback object.
            dataObjects -- Array of Omero IObjects
            options -- Parameters as above.
            current -- The Current object for the invocation.
            """
            pass

        def unlink_async(self, _cb, links, options, current=None):
            """
            Removes links between OmeroDataObjects e.g Project-Dataset,
            Dataset-Image
            Note that the objects themselves aren't deleted, only the
            Link objects.
            Arguments:
            _cb -- The asynchronous callback object.
            links -- Not null.
            options -- Parameters as above.
            current -- The Current object for the invocation.
            """
            pass

        def link_async(self, _cb, links, options, current=None):
            """
            Convenience method for creating links. Functionality also
            available from {@code createDataObject}
            Arguments:
            _cb -- The asynchronous callback object.
            links -- Array of links to be created.
            options -- Parameters as above.
            current -- The Current object for the invocation.
            """
            pass

        def updateDataObject_async(self, _cb, obj, options, current=None):
            """
            Updates a data object.
            To link or unlink objects to the specified object, we
            should call the methods link or unlink. TODO Or do we use
            for example dataset.setProjects(set of projects) to add.
            Link has to be set as follows dataset&rarr;project and
            project&rarr;dataset.
            Alternatively, you can make sure that the collection is
            exactly how it should be in the database. If you
            can't guarantee this, it's best to send all your
            collections back as null
            Arguments:
            _cb -- The asynchronous callback object.
            obj -- Pojos-based IObject. Supported: Project, Dataset, Annotation, Group, Experimenter.
            options -- Parameters as above.
            current -- The Current object for the invocation.
            """
            pass

        def updateDataObjects_async(self, _cb, objs, options, current=None):
            """
            Convenience method to save network calls. Loops over the
            array of IObjects calling {@code updateDataObject}.
            Arguments:
            _cb -- The asynchronous callback object.
            objs -- 
            options -- Parameters as above.
            current -- The Current object for the invocation.
            """
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.api._t_IContainer)

        __repr__ = __str__

    _M_omero.api.IContainerPrx = Ice.createTempClass()
    class IContainerPrx(_M_omero.api.ServiceInterfacePrx):

        """
        Retrieves hierarchy trees rooted by a given node (unless
        orphan is specified -- See below)
        This method also retrieves the Experimenters linked to the
        objects in the tree. Similarly, all Images will be linked
        to their Pixel objects if included.
        Note that objects are never duplicated. For example, if an
        Experimenter owns all the objects in the retrieved tree,
        then those objects will be linked to the same
        instance of omero.model.Experimenter. Or if an
        Image is contained in more than one Dataset in the
        retrieved tree, then all enclosing
        omero.model.Dataset objects will point
        to the same omero.model.Image object. And
        so on.
        Arguments:
        rootType -- The type of the root node. Can be omero.model.Project, omero.model.Dataset, omero.model.Screen or omero.model.Plate. Cannot be null.
        rootIds -- The ids of the root nodes. Can be null if an Experimenter is specified in options, otherwise an Exception is thrown to prevent all images in the entire database from being downloaded.
        options -- Parameters as above. annotator, leaves, orphan, acquisition data used. acquisition data is only relevant for images and taken into account if the images are loaded. If rootNodeIds==null, experimenter|group must be set and filtering will be applied at the Class-level; e.g. to retrieve a user's Projects, or user's Datasets. If rootNodeIds!=null, the result will be filtered by the experimenter|group at the Image and intermediate levels if available. Due to the amount of data potentially linked a Screen/Plate, the leaves option is not taken into account when the root node is a omero.model.Screen. orphan implies that objects which are not contained in an object of rootNodeType should also be returned.
        _ctx -- The request context for the invocation.
        Returns: a set of hierarchy trees. The requested node as root and all of its descendants. The type of the returned value will be rootNodeType, unless orphan is specified in which case objects of type rootNodeType and below may be returned.
        """
        def loadContainerHierarchy(self, rootType, rootIds, options, _ctx=None):
            return _M_omero.api.IContainer._op_loadContainerHierarchy.invoke(self, ((rootType, rootIds, options), _ctx))

        """
        Retrieves hierarchy trees rooted by a given node (unless
        orphan is specified -- See below)
        This method also retrieves the Experimenters linked to the
        objects in the tree. Similarly, all Images will be linked
        to their Pixel objects if included.
        Note that objects are never duplicated. For example, if an
        Experimenter owns all the objects in the retrieved tree,
        then those objects will be linked to the same
        instance of omero.model.Experimenter. Or if an
        Image is contained in more than one Dataset in the
        retrieved tree, then all enclosing
        omero.model.Dataset objects will point
        to the same omero.model.Image object. And
        so on.
        Arguments:
        rootType -- The type of the root node. Can be omero.model.Project, omero.model.Dataset, omero.model.Screen or omero.model.Plate. Cannot be null.
        rootIds -- The ids of the root nodes. Can be null if an Experimenter is specified in options, otherwise an Exception is thrown to prevent all images in the entire database from being downloaded.
        options -- Parameters as above. annotator, leaves, orphan, acquisition data used. acquisition data is only relevant for images and taken into account if the images are loaded. If rootNodeIds==null, experimenter|group must be set and filtering will be applied at the Class-level; e.g. to retrieve a user's Projects, or user's Datasets. If rootNodeIds!=null, the result will be filtered by the experimenter|group at the Image and intermediate levels if available. Due to the amount of data potentially linked a Screen/Plate, the leaves option is not taken into account when the root node is a omero.model.Screen. orphan implies that objects which are not contained in an object of rootNodeType should also be returned.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_loadContainerHierarchy(self, rootType, rootIds, options, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IContainer._op_loadContainerHierarchy.begin(self, ((rootType, rootIds, options), _response, _ex, _sent, _ctx))

        """
        Retrieves hierarchy trees rooted by a given node (unless
        orphan is specified -- See below)
        This method also retrieves the Experimenters linked to the
        objects in the tree. Similarly, all Images will be linked
        to their Pixel objects if included.
        Note that objects are never duplicated. For example, if an
        Experimenter owns all the objects in the retrieved tree,
        then those objects will be linked to the same
        instance of omero.model.Experimenter. Or if an
        Image is contained in more than one Dataset in the
        retrieved tree, then all enclosing
        omero.model.Dataset objects will point
        to the same omero.model.Image object. And
        so on.
        Arguments:
        rootType -- The type of the root node. Can be omero.model.Project, omero.model.Dataset, omero.model.Screen or omero.model.Plate. Cannot be null.
        rootIds -- The ids of the root nodes. Can be null if an Experimenter is specified in options, otherwise an Exception is thrown to prevent all images in the entire database from being downloaded.
        options -- Parameters as above. annotator, leaves, orphan, acquisition data used. acquisition data is only relevant for images and taken into account if the images are loaded. If rootNodeIds==null, experimenter|group must be set and filtering will be applied at the Class-level; e.g. to retrieve a user's Projects, or user's Datasets. If rootNodeIds!=null, the result will be filtered by the experimenter|group at the Image and intermediate levels if available. Due to the amount of data potentially linked a Screen/Plate, the leaves option is not taken into account when the root node is a omero.model.Screen. orphan implies that objects which are not contained in an object of rootNodeType should also be returned.
        Returns: a set of hierarchy trees. The requested node as root and all of its descendants. The type of the returned value will be rootNodeType, unless orphan is specified in which case objects of type rootNodeType and below may be returned.
        """
        def end_loadContainerHierarchy(self, _r):
            return _M_omero.api.IContainer._op_loadContainerHierarchy.end(self, _r)

        """
        Retrieves hierarchy trees in various hierarchies that
        contain the specified Images.
        This method will look for all the containers containing the
        specified Images and then for all containers containing
        those containers and on up the container hierarchy.
        This method returns a Set with all root nodes
        that were
        found. Every root node is linked to the found objects and
        so on until the leaf nodes, which are
        omero.model.Image objects. Note that the type of any
        root node in the returned set can be the given
        rootNodeType, any of its containees or an
        omero.model.Image.
        For example, say that you pass in the ids of six Images:
        i1, i2, i3, i4, i5, i6.
        If the P/D/I hierarchy in the DB looks like this:
        |                  __p1__                     
        |                 /      \                    
        |               _d1_    _d2_      d3          
        |              /    \  /    \     |           
        |             i1     i2     i3    i4    i5  i6
        Then the returned set will contain
        p1, d3, i5, i6. All objects will be properly
        linked up.
        Finally, this method will only retrieve the nodes
        that are connected in a tree to the specified leaf image
        nodes. Back to the previous example, if d1
        contained image img500, then the returned
        object would not contain img500. In a
        similar way, if p1 contained
        ds300 and this dataset weren't linked to any of
        the i1, i2, i3, i4, i5, i6  images, then
        ds300 would not be part of the returned
        tree rooted by p1.
        Arguments:
        rootType -- top-most type which will be searched for Can be omero.model.Project. Not null.
        imageIds -- Contains the ids of the Images that sit at the bottom of the trees. Not null.
        options -- Parameters as above. annotator used. experimenter|group may be applied at the top-level only or at each level in the hierarchy, but will not apply to the leaf (Image) level.
        _ctx -- The request context for the invocation.
        Returns: A Set with all root nodes that were found.
        """
        def findContainerHierarchies(self, rootType, imageIds, options, _ctx=None):
            return _M_omero.api.IContainer._op_findContainerHierarchies.invoke(self, ((rootType, imageIds, options), _ctx))

        """
        Retrieves hierarchy trees in various hierarchies that
        contain the specified Images.
        This method will look for all the containers containing the
        specified Images and then for all containers containing
        those containers and on up the container hierarchy.
        This method returns a Set with all root nodes
        that were
        found. Every root node is linked to the found objects and
        so on until the leaf nodes, which are
        omero.model.Image objects. Note that the type of any
        root node in the returned set can be the given
        rootNodeType, any of its containees or an
        omero.model.Image.
        For example, say that you pass in the ids of six Images:
        i1, i2, i3, i4, i5, i6.
        If the P/D/I hierarchy in the DB looks like this:
        |                  __p1__                     
        |                 /      \                    
        |               _d1_    _d2_      d3          
        |              /    \  /    \     |           
        |             i1     i2     i3    i4    i5  i6
        Then the returned set will contain
        p1, d3, i5, i6. All objects will be properly
        linked up.
        Finally, this method will only retrieve the nodes
        that are connected in a tree to the specified leaf image
        nodes. Back to the previous example, if d1
        contained image img500, then the returned
        object would not contain img500. In a
        similar way, if p1 contained
        ds300 and this dataset weren't linked to any of
        the i1, i2, i3, i4, i5, i6  images, then
        ds300 would not be part of the returned
        tree rooted by p1.
        Arguments:
        rootType -- top-most type which will be searched for Can be omero.model.Project. Not null.
        imageIds -- Contains the ids of the Images that sit at the bottom of the trees. Not null.
        options -- Parameters as above. annotator used. experimenter|group may be applied at the top-level only or at each level in the hierarchy, but will not apply to the leaf (Image) level.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_findContainerHierarchies(self, rootType, imageIds, options, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IContainer._op_findContainerHierarchies.begin(self, ((rootType, imageIds, options), _response, _ex, _sent, _ctx))

        """
        Retrieves hierarchy trees in various hierarchies that
        contain the specified Images.
        This method will look for all the containers containing the
        specified Images and then for all containers containing
        those containers and on up the container hierarchy.
        This method returns a Set with all root nodes
        that were
        found. Every root node is linked to the found objects and
        so on until the leaf nodes, which are
        omero.model.Image objects. Note that the type of any
        root node in the returned set can be the given
        rootNodeType, any of its containees or an
        omero.model.Image.
        For example, say that you pass in the ids of six Images:
        i1, i2, i3, i4, i5, i6.
        If the P/D/I hierarchy in the DB looks like this:
        |                  __p1__                     
        |                 /      \                    
        |               _d1_    _d2_      d3          
        |              /    \  /    \     |           
        |             i1     i2     i3    i4    i5  i6
        Then the returned set will contain
        p1, d3, i5, i6. All objects will be properly
        linked up.
        Finally, this method will only retrieve the nodes
        that are connected in a tree to the specified leaf image
        nodes. Back to the previous example, if d1
        contained image img500, then the returned
        object would not contain img500. In a
        similar way, if p1 contained
        ds300 and this dataset weren't linked to any of
        the i1, i2, i3, i4, i5, i6  images, then
        ds300 would not be part of the returned
        tree rooted by p1.
        Arguments:
        rootType -- top-most type which will be searched for Can be omero.model.Project. Not null.
        imageIds -- Contains the ids of the Images that sit at the bottom of the trees. Not null.
        options -- Parameters as above. annotator used. experimenter|group may be applied at the top-level only or at each level in the hierarchy, but will not apply to the leaf (Image) level.
        Returns: A Set with all root nodes that were found.
        """
        def end_findContainerHierarchies(self, _r):
            return _M_omero.api.IContainer._op_findContainerHierarchies.end(self, _r)

        """
        Retrieve a user's (or all users') images within any given
        container. For example, all images in project, applying
        temporal filtering or pagination.
        Arguments:
        rootType -- A Class which will have its hierarchy searched for Images. Not null.
        rootIds -- A set of ids of type rootNodeType Not null.
        options -- Parameters as above. No notion of leaves. experimenter|group apply at the Image level. OPTIONS: - startTime and/or endTime should be Timestamp.valueOf("YYYY-MM-DD hh:mm:ss.ms"); limit and offset are applied at the Image-level. That is, calling with Dataset.class, limit == 10 and offset == 0 will first perform one query to get an effective set of rootNodeIds, then getImages will be called with an effective rootNodeType of Image.class and the new ids. acquisition data is only relevant for images.
        _ctx -- The request context for the invocation.
        Returns: A set of images.
        """
        def getImages(self, rootType, rootIds, options, _ctx=None):
            return _M_omero.api.IContainer._op_getImages.invoke(self, ((rootType, rootIds, options), _ctx))

        """
        Retrieve a user's (or all users') images within any given
        container. For example, all images in project, applying
        temporal filtering or pagination.
        Arguments:
        rootType -- A Class which will have its hierarchy searched for Images. Not null.
        rootIds -- A set of ids of type rootNodeType Not null.
        options -- Parameters as above. No notion of leaves. experimenter|group apply at the Image level. OPTIONS: - startTime and/or endTime should be Timestamp.valueOf("YYYY-MM-DD hh:mm:ss.ms"); limit and offset are applied at the Image-level. That is, calling with Dataset.class, limit == 10 and offset == 0 will first perform one query to get an effective set of rootNodeIds, then getImages will be called with an effective rootNodeType of Image.class and the new ids. acquisition data is only relevant for images.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getImages(self, rootType, rootIds, options, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IContainer._op_getImages.begin(self, ((rootType, rootIds, options), _response, _ex, _sent, _ctx))

        """
        Retrieve a user's (or all users') images within any given
        container. For example, all images in project, applying
        temporal filtering or pagination.
        Arguments:
        rootType -- A Class which will have its hierarchy searched for Images. Not null.
        rootIds -- A set of ids of type rootNodeType Not null.
        options -- Parameters as above. No notion of leaves. experimenter|group apply at the Image level. OPTIONS: - startTime and/or endTime should be Timestamp.valueOf("YYYY-MM-DD hh:mm:ss.ms"); limit and offset are applied at the Image-level. That is, calling with Dataset.class, limit == 10 and offset == 0 will first perform one query to get an effective set of rootNodeIds, then getImages will be called with an effective rootNodeType of Image.class and the new ids. acquisition data is only relevant for images.
        Returns: A set of images.
        """
        def end_getImages(self, _r):
            return _M_omero.api.IContainer._op_getImages.end(self, _r)

        """
        Retrieves a user's images.
        Arguments:
        options -- Parameters as above. No notion of leaves. experimenter|group apply at the Image level and must be present.
        _ctx -- The request context for the invocation.
        Returns: A set of images.
        """
        def getUserImages(self, options, _ctx=None):
            return _M_omero.api.IContainer._op_getUserImages.invoke(self, ((options, ), _ctx))

        """
        Retrieves a user's images.
        Arguments:
        options -- Parameters as above. No notion of leaves. experimenter|group apply at the Image level and must be present.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getUserImages(self, options, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IContainer._op_getUserImages.begin(self, ((options, ), _response, _ex, _sent, _ctx))

        """
        Retrieves a user's images.
        Arguments:
        options -- Parameters as above. No notion of leaves. experimenter|group apply at the Image level and must be present.
        Returns: A set of images.
        """
        def end_getUserImages(self, _r):
            return _M_omero.api.IContainer._op_getUserImages.end(self, _r)

        """
        Retrieves images by options.
        Arguments:
        options -- Parameters as above. No notion of leaves. experimenter|group apply at the Image level and must be present. OPTIONS: - startTime and/or endTime should be Timestamp.valueOf("YYYY-MM-DD hh:mm:ss.ms"). acquisition data is only relevant for images.
        _ctx -- The request context for the invocation.
        Returns: A set of images.
        """
        def getImagesByOptions(self, options, _ctx=None):
            return _M_omero.api.IContainer._op_getImagesByOptions.invoke(self, ((options, ), _ctx))

        """
        Retrieves images by options.
        Arguments:
        options -- Parameters as above. No notion of leaves. experimenter|group apply at the Image level and must be present. OPTIONS: - startTime and/or endTime should be Timestamp.valueOf("YYYY-MM-DD hh:mm:ss.ms"). acquisition data is only relevant for images.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getImagesByOptions(self, options, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IContainer._op_getImagesByOptions.begin(self, ((options, ), _response, _ex, _sent, _ctx))

        """
        Retrieves images by options.
        Arguments:
        options -- Parameters as above. No notion of leaves. experimenter|group apply at the Image level and must be present. OPTIONS: - startTime and/or endTime should be Timestamp.valueOf("YYYY-MM-DD hh:mm:ss.ms"). acquisition data is only relevant for images.
        Returns: A set of images.
        """
        def end_getImagesByOptions(self, _r):
            return _M_omero.api.IContainer._op_getImagesByOptions.end(self, _r)

        """
        Given a list of IDs of certain entity types, calculates
        which filesets are split such that a non-empty proper
        subset of their images are referenced, directly or
        indirectly, as being included. The return value lists both
        the fileset IDs and the image IDs in ascending order,
        the image ID lists separated by if they were included.
        Warning: following discussion in trac ticket 11019 the
        return type may be changed.
        Arguments:
        included -- the entities included
        options -- parameters, presently ignored
        _ctx -- The request context for the invocation.
        Returns: the partially included filesets
        """
        def getImagesBySplitFilesets(self, included, options, _ctx=None):
            return _M_omero.api.IContainer._op_getImagesBySplitFilesets.invoke(self, ((included, options), _ctx))

        """
        Given a list of IDs of certain entity types, calculates
        which filesets are split such that a non-empty proper
        subset of their images are referenced, directly or
        indirectly, as being included. The return value lists both
        the fileset IDs and the image IDs in ascending order,
        the image ID lists separated by if they were included.
        Warning: following discussion in trac ticket 11019 the
        return type may be changed.
        Arguments:
        included -- the entities included
        options -- parameters, presently ignored
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getImagesBySplitFilesets(self, included, options, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IContainer._op_getImagesBySplitFilesets.begin(self, ((included, options), _response, _ex, _sent, _ctx))

        """
        Given a list of IDs of certain entity types, calculates
        which filesets are split such that a non-empty proper
        subset of their images are referenced, directly or
        indirectly, as being included. The return value lists both
        the fileset IDs and the image IDs in ascending order,
        the image ID lists separated by if they were included.
        Warning: following discussion in trac ticket 11019 the
        return type may be changed.
        Arguments:
        included -- the entities included
        options -- parameters, presently ignored
        Returns: the partially included filesets
        """
        def end_getImagesBySplitFilesets(self, _r):
            return _M_omero.api.IContainer._op_getImagesBySplitFilesets.end(self, _r)

        """
        Counts the number of members in a collection for a given
        object. For example, if you wanted to retrieve the number
        of Images contained in a Dataset you would pass TODO.
        Arguments:
        type -- The fully-qualified classname of the object to be tested
        property -- Name of the property on that class, omitting getters and setters.
        ids -- Set of Longs, the ids of the objects to test
        options -- Parameters. Unused.
        _ctx -- The request context for the invocation.
        Returns: A map from id integer to count integer
        """
        def getCollectionCount(self, type, property, ids, options, _ctx=None):
            return _M_omero.api.IContainer._op_getCollectionCount.invoke(self, ((type, property, ids, options), _ctx))

        """
        Counts the number of members in a collection for a given
        object. For example, if you wanted to retrieve the number
        of Images contained in a Dataset you would pass TODO.
        Arguments:
        type -- The fully-qualified classname of the object to be tested
        property -- Name of the property on that class, omitting getters and setters.
        ids -- Set of Longs, the ids of the objects to test
        options -- Parameters. Unused.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getCollectionCount(self, type, property, ids, options, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IContainer._op_getCollectionCount.begin(self, ((type, property, ids, options), _response, _ex, _sent, _ctx))

        """
        Counts the number of members in a collection for a given
        object. For example, if you wanted to retrieve the number
        of Images contained in a Dataset you would pass TODO.
        Arguments:
        type -- The fully-qualified classname of the object to be tested
        property -- Name of the property on that class, omitting getters and setters.
        ids -- Set of Longs, the ids of the objects to test
        options -- Parameters. Unused.
        Returns: A map from id integer to count integer
        """
        def end_getCollectionCount(self, _r):
            return _M_omero.api.IContainer._op_getCollectionCount.end(self, _r)

        """
        Retrieves a collection with all members initialized
        (loaded). This is useful when a collection has been
        nulled in a previous query.
        Arguments:
        obj -- Can be unloaded.
        collectionName -- public static final String from the IObject.class
        options -- Parameters. Unused.
        _ctx -- The request context for the invocation.
        Returns: An initialized collection.
        """
        def retrieveCollection(self, obj, collectionName, options, _ctx=None):
            return _M_omero.api.IContainer._op_retrieveCollection.invoke(self, ((obj, collectionName, options), _ctx))

        """
        Retrieves a collection with all members initialized
        (loaded). This is useful when a collection has been
        nulled in a previous query.
        Arguments:
        obj -- Can be unloaded.
        collectionName -- public static final String from the IObject.class
        options -- Parameters. Unused.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_retrieveCollection(self, obj, collectionName, options, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IContainer._op_retrieveCollection.begin(self, ((obj, collectionName, options), _response, _ex, _sent, _ctx))

        """
        Retrieves a collection with all members initialized
        (loaded). This is useful when a collection has been
        nulled in a previous query.
        Arguments:
        obj -- Can be unloaded.
        collectionName -- public static final String from the IObject.class
        options -- Parameters. Unused.
        Returns: An initialized collection.
        """
        def end_retrieveCollection(self, _r):
            return _M_omero.api.IContainer._op_retrieveCollection.end(self, _r)

        """
        Creates the specified data object.
        A placeholder parent object is created if the data object
        is to be put in a collection.
        For example, if the object is a Dataset, we
        first create a Project as parent then we set
        the Dataset parent as follows: 
        //pseudo-code TODO
        Project p = new Project(id,false);
        dataset.addProject(p);
        then for each parent relationship a DataObject
        ome.model.ILink is created.
        Arguments:
        obj -- IObject. Supported: Project, Dataset, Annotation, Group, Experimenter. Not null.
        options -- Parameters as above.
        _ctx -- The request context for the invocation.
        Returns: the created object
        """
        def createDataObject(self, obj, options, _ctx=None):
            return _M_omero.api.IContainer._op_createDataObject.invoke(self, ((obj, options), _ctx))

        """
        Creates the specified data object.
        A placeholder parent object is created if the data object
        is to be put in a collection.
        For example, if the object is a Dataset, we
        first create a Project as parent then we set
        the Dataset parent as follows: 
        //pseudo-code TODO
        Project p = new Project(id,false);
        dataset.addProject(p);
        then for each parent relationship a DataObject
        ome.model.ILink is created.
        Arguments:
        obj -- IObject. Supported: Project, Dataset, Annotation, Group, Experimenter. Not null.
        options -- Parameters as above.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_createDataObject(self, obj, options, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IContainer._op_createDataObject.begin(self, ((obj, options), _response, _ex, _sent, _ctx))

        """
        Creates the specified data object.
        A placeholder parent object is created if the data object
        is to be put in a collection.
        For example, if the object is a Dataset, we
        first create a Project as parent then we set
        the Dataset parent as follows: 
        //pseudo-code TODO
        Project p = new Project(id,false);
        dataset.addProject(p);
        then for each parent relationship a DataObject
        ome.model.ILink is created.
        Arguments:
        obj -- IObject. Supported: Project, Dataset, Annotation, Group, Experimenter. Not null.
        options -- Parameters as above.
        Returns: the created object
        """
        def end_createDataObject(self, _r):
            return _M_omero.api.IContainer._op_createDataObject.end(self, _r)

        """
        Convenience method to save network calls. Loops over the
        array of IObjects calling {@code createDataObject}.
        Arguments:
        dataObjects -- Array of Omero IObjects
        options -- Parameters as above.
        _ctx -- The request context for the invocation.
        """
        def createDataObjects(self, dataObjects, options, _ctx=None):
            return _M_omero.api.IContainer._op_createDataObjects.invoke(self, ((dataObjects, options), _ctx))

        """
        Convenience method to save network calls. Loops over the
        array of IObjects calling {@code createDataObject}.
        Arguments:
        dataObjects -- Array of Omero IObjects
        options -- Parameters as above.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_createDataObjects(self, dataObjects, options, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IContainer._op_createDataObjects.begin(self, ((dataObjects, options), _response, _ex, _sent, _ctx))

        """
        Convenience method to save network calls. Loops over the
        array of IObjects calling {@code createDataObject}.
        Arguments:
        dataObjects -- Array of Omero IObjects
        options -- Parameters as above.
        """
        def end_createDataObjects(self, _r):
            return _M_omero.api.IContainer._op_createDataObjects.end(self, _r)

        """
        Removes links between OmeroDataObjects e.g Project-Dataset,
        Dataset-Image
        Note that the objects themselves aren't deleted, only the
        Link objects.
        Arguments:
        links -- Not null.
        options -- Parameters as above.
        _ctx -- The request context for the invocation.
        """
        def unlink(self, links, options, _ctx=None):
            return _M_omero.api.IContainer._op_unlink.invoke(self, ((links, options), _ctx))

        """
        Removes links between OmeroDataObjects e.g Project-Dataset,
        Dataset-Image
        Note that the objects themselves aren't deleted, only the
        Link objects.
        Arguments:
        links -- Not null.
        options -- Parameters as above.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_unlink(self, links, options, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IContainer._op_unlink.begin(self, ((links, options), _response, _ex, _sent, _ctx))

        """
        Removes links between OmeroDataObjects e.g Project-Dataset,
        Dataset-Image
        Note that the objects themselves aren't deleted, only the
        Link objects.
        Arguments:
        links -- Not null.
        options -- Parameters as above.
        """
        def end_unlink(self, _r):
            return _M_omero.api.IContainer._op_unlink.end(self, _r)

        """
        Convenience method for creating links. Functionality also
        available from {@code createDataObject}
        Arguments:
        links -- Array of links to be created.
        options -- Parameters as above.
        _ctx -- The request context for the invocation.
        Returns: the created links
        """
        def link(self, links, options, _ctx=None):
            return _M_omero.api.IContainer._op_link.invoke(self, ((links, options), _ctx))

        """
        Convenience method for creating links. Functionality also
        available from {@code createDataObject}
        Arguments:
        links -- Array of links to be created.
        options -- Parameters as above.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_link(self, links, options, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IContainer._op_link.begin(self, ((links, options), _response, _ex, _sent, _ctx))

        """
        Convenience method for creating links. Functionality also
        available from {@code createDataObject}
        Arguments:
        links -- Array of links to be created.
        options -- Parameters as above.
        Returns: the created links
        """
        def end_link(self, _r):
            return _M_omero.api.IContainer._op_link.end(self, _r)

        """
        Updates a data object.
        To link or unlink objects to the specified object, we
        should call the methods link or unlink. TODO Or do we use
        for example dataset.setProjects(set of projects) to add.
        Link has to be set as follows dataset&rarr;project and
        project&rarr;dataset.
        Alternatively, you can make sure that the collection is
        exactly how it should be in the database. If you
        can't guarantee this, it's best to send all your
        collections back as null
        Arguments:
        obj -- Pojos-based IObject. Supported: Project, Dataset, Annotation, Group, Experimenter.
        options -- Parameters as above.
        _ctx -- The request context for the invocation.
        Returns: created data object
        """
        def updateDataObject(self, obj, options, _ctx=None):
            return _M_omero.api.IContainer._op_updateDataObject.invoke(self, ((obj, options), _ctx))

        """
        Updates a data object.
        To link or unlink objects to the specified object, we
        should call the methods link or unlink. TODO Or do we use
        for example dataset.setProjects(set of projects) to add.
        Link has to be set as follows dataset&rarr;project and
        project&rarr;dataset.
        Alternatively, you can make sure that the collection is
        exactly how it should be in the database. If you
        can't guarantee this, it's best to send all your
        collections back as null
        Arguments:
        obj -- Pojos-based IObject. Supported: Project, Dataset, Annotation, Group, Experimenter.
        options -- Parameters as above.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_updateDataObject(self, obj, options, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IContainer._op_updateDataObject.begin(self, ((obj, options), _response, _ex, _sent, _ctx))

        """
        Updates a data object.
        To link or unlink objects to the specified object, we
        should call the methods link or unlink. TODO Or do we use
        for example dataset.setProjects(set of projects) to add.
        Link has to be set as follows dataset&rarr;project and
        project&rarr;dataset.
        Alternatively, you can make sure that the collection is
        exactly how it should be in the database. If you
        can't guarantee this, it's best to send all your
        collections back as null
        Arguments:
        obj -- Pojos-based IObject. Supported: Project, Dataset, Annotation, Group, Experimenter.
        options -- Parameters as above.
        Returns: created data object
        """
        def end_updateDataObject(self, _r):
            return _M_omero.api.IContainer._op_updateDataObject.end(self, _r)

        """
        Convenience method to save network calls. Loops over the
        array of IObjects calling {@code updateDataObject}.
        Arguments:
        objs -- 
        options -- Parameters as above.
        _ctx -- The request context for the invocation.
        Returns: created data objects.
        """
        def updateDataObjects(self, objs, options, _ctx=None):
            return _M_omero.api.IContainer._op_updateDataObjects.invoke(self, ((objs, options), _ctx))

        """
        Convenience method to save network calls. Loops over the
        array of IObjects calling {@code updateDataObject}.
        Arguments:
        objs -- 
        options -- Parameters as above.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_updateDataObjects(self, objs, options, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IContainer._op_updateDataObjects.begin(self, ((objs, options), _response, _ex, _sent, _ctx))

        """
        Convenience method to save network calls. Loops over the
        array of IObjects calling {@code updateDataObject}.
        Arguments:
        objs -- 
        options -- Parameters as above.
        Returns: created data objects.
        """
        def end_updateDataObjects(self, _r):
            return _M_omero.api.IContainer._op_updateDataObjects.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.api.IContainerPrx.ice_checkedCast(proxy, '::omero::api::IContainer', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.api.IContainerPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::api::IContainer'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.api._t_IContainerPrx = IcePy.defineProxy('::omero::api::IContainer', IContainerPrx)

    _M_omero.api._t_IContainer = IcePy.defineClass('::omero::api::IContainer', IContainer, -1, (), True, False, None, (_M_omero.api._t_ServiceInterface,), ())
    IContainer._ice_type = _M_omero.api._t_IContainer

    IContainer._op_loadContainerHierarchy = IcePy.Operation('loadContainerHierarchy', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_string, False, 0), ((), _M_omero.sys._t_LongList, False, 0), ((), _M_omero.sys._t_Parameters, False, 0)), (), ((), _M_omero.api._t_IObjectList, False, 0), (_M_omero._t_ServerError,))
    IContainer._op_findContainerHierarchies = IcePy.Operation('findContainerHierarchies', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_string, False, 0), ((), _M_omero.sys._t_LongList, False, 0), ((), _M_omero.sys._t_Parameters, False, 0)), (), ((), _M_omero.api._t_IObjectList, False, 0), (_M_omero._t_ServerError,))
    IContainer._op_getImages = IcePy.Operation('getImages', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_string, False, 0), ((), _M_omero.sys._t_LongList, False, 0), ((), _M_omero.sys._t_Parameters, False, 0)), (), ((), _M_omero.api._t_ImageList, False, 0), (_M_omero._t_ServerError,))
    IContainer._op_getUserImages = IcePy.Operation('getUserImages', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), _M_omero.sys._t_Parameters, False, 0),), (), ((), _M_omero.api._t_ImageList, False, 0), (_M_omero._t_ServerError,))
    IContainer._op_getImagesByOptions = IcePy.Operation('getImagesByOptions', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), _M_omero.sys._t_Parameters, False, 0),), (), ((), _M_omero.api._t_ImageList, False, 0), (_M_omero._t_ServerError,))
    IContainer._op_getImagesBySplitFilesets = IcePy.Operation('getImagesBySplitFilesets', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), _M_omero.api._t_StringLongListMap, False, 0), ((), _M_omero.sys._t_Parameters, False, 0)), (), ((), _M_omero.api._t_IdBooleanLongListMapMap, False, 0), (_M_omero._t_ServerError,))
    IContainer._op_getCollectionCount = IcePy.Operation('getCollectionCount', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_string, False, 0), ((), IcePy._t_string, False, 0), ((), _M_omero.sys._t_LongList, False, 0), ((), _M_omero.sys._t_Parameters, False, 0)), (), ((), _M_omero.sys._t_CountMap, False, 0), (_M_omero._t_ServerError,))
    IContainer._op_retrieveCollection = IcePy.Operation('retrieveCollection', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), _M_omero.model._t_IObject, False, 0), ((), IcePy._t_string, False, 0), ((), _M_omero.sys._t_Parameters, False, 0)), (), ((), _M_omero.api._t_IObjectList, False, 0), (_M_omero._t_ServerError,))
    IContainer._op_createDataObject = IcePy.Operation('createDataObject', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), _M_omero.model._t_IObject, False, 0), ((), _M_omero.sys._t_Parameters, False, 0)), (), ((), _M_omero.model._t_IObject, False, 0), (_M_omero._t_ServerError,))
    IContainer._op_createDataObjects = IcePy.Operation('createDataObjects', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), _M_omero.api._t_IObjectList, False, 0), ((), _M_omero.sys._t_Parameters, False, 0)), (), ((), _M_omero.api._t_IObjectList, False, 0), (_M_omero._t_ServerError,))
    IContainer._op_unlink = IcePy.Operation('unlink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), _M_omero.api._t_IObjectList, False, 0), ((), _M_omero.sys._t_Parameters, False, 0)), (), None, (_M_omero._t_ServerError,))
    IContainer._op_link = IcePy.Operation('link', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), _M_omero.api._t_IObjectList, False, 0), ((), _M_omero.sys._t_Parameters, False, 0)), (), ((), _M_omero.api._t_IObjectList, False, 0), (_M_omero._t_ServerError,))
    IContainer._op_updateDataObject = IcePy.Operation('updateDataObject', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), _M_omero.model._t_IObject, False, 0), ((), _M_omero.sys._t_Parameters, False, 0)), (), ((), _M_omero.model._t_IObject, False, 0), (_M_omero._t_ServerError,))
    IContainer._op_updateDataObjects = IcePy.Operation('updateDataObjects', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), _M_omero.api._t_IObjectList, False, 0), ((), _M_omero.sys._t_Parameters, False, 0)), (), ((), _M_omero.api._t_IObjectList, False, 0), (_M_omero._t_ServerError,))

    _M_omero.api.IContainer = IContainer
    del IContainer

    _M_omero.api.IContainerPrx = IContainerPrx
    del IContainerPrx

# End of module omero.api

__name__ = 'omero'

# End of module omero



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2006-2009 University of Dundee. All rights reserved.
#
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

# @author  William Moore &nbsp;&nbsp;&nbsp;&nbsp;
# <a href="mailto:will@lifesci.dundee.ac.uk">will@lifesci.dundee.ac.uk</a>
# @author  Jean-Marie Burel &nbsp;&nbsp;&nbsp;&nbsp;
# <a href="mailto:j.burel@dundee.ac.uk">j.burel@dundee.ac.uk</a>
# @author Donald MacDonald &nbsp;&nbsp;&nbsp;&nbsp;
# <a href="mailto:donald@lifesci.dundee.ac.uk">donald@lifesci.dundee.ac.uk</a>

"""
A collection of utility methods based on the Python Imaging Library (PIL)
used for making figures.
"""
from __future__ import division

from future import standard_library
standard_library.install_aliases()
from past.utils import old_div
try:
    from PIL import Image, ImageDraw, ImageFont  # see ticket:2597
except ImportError:
    import Image
    import ImageDraw
    import ImageFont  # see ticket:2597

import os.path
import omero.gateway
import io
from omero.rtypes import rint
import warnings

GATEWAYPATH = omero.gateway.THISPATH


def getFont(fontsize):
    """
    Returns a PIL ImageFont Sans-serif true-type font of the specified size
    or a pre-compiled font of fixed size if the ttf font is not found

    :param fontsize:	The size of the font you want
    :return: 	A PIL Font
    """
    warnings.warn(
        "This module is deprecated as of OMERO 5.3.0", DeprecationWarning)
    fontPath = os.path.join(GATEWAYPATH, "pilfonts", "FreeSans.ttf")
    try:
        font = ImageFont.truetype(fontPath, fontsize)
    except:
        font = ImageFont.load('%s/pilfonts/B%0.2d.pil' % (GATEWAYPATH, 24))
    return font


def pasteImage(image, canvas, x, y):
    """
    Pastes the image onto the canvas at the specified coordinates
    Image and canvas are instances of PIL 'Image'

    :param image:		The PIL image to be pasted. Image
    :param canvas:		The PIL image on which to paste. Image
    :param x:			X coordinate (left) to paste
    :param y: 			Y coordinate (top) to paste
    """
    warnings.warn(
        "This module is deprecated as of OMERO 5.3.0", DeprecationWarning)
    xRight = image.size[0] + x
    yBottom = image.size[1] + y
    # make a tuple of topleft-x, topleft-y, bottomRight-x, bottomRight-y
    pasteBox = (x, y, xRight, yBottom)
    canvas.paste(image, pasteBox)


def getThumbnail(thumbnailStore, pixelsId, length):
    """
    Returns a thumbnail (as string) from the pixelsId,
    the longest side is 'length'

    :param thumbnailStore: 	The Omero thumbnail store
    :param pixelsId:		The ID of the pixels. long
    :param length:			Length of longest side. int
    :return:				The thumbnail as a String,
                            or None if not found (invalid image)
    """
    warnings.warn(
        "This module is deprecated as of OMERO 5.3.0", DeprecationWarning)
    if not thumbnailStore.setPixelsId(pixelsId):
        thumbnailStore.needDefaults()
        thumbnailStore.setPixelsId(pixelsId)
    try:
        # returns string (api says Ice::ByteSeq)
        return thumbnailStore.getThumbnailByLongestSide(rint(length))
    except:
        return None


def getThumbnailSet(thumbnailStore, length, pixelIds):
    """
    Returns map of thumbnails whose keys are the pixels id
    and the values are the image, the longest side is 'length'

    :param thumbnailStore: 	The Omero thumbnail store
    :param pixelIds:		The collection of pixels ID.
    :param length:		    Length of longest side. int
    :return:                See above
    """
    warnings.warn(
        "This module is deprecated as of OMERO 5.3.0", DeprecationWarning)
    try:
        # returns string (api says Ice::ByteSeq)
        return thumbnailStore.getThumbnailByLongestSideSet(
            rint(length), pixelIds)
    except:
        return None


def paintThumbnailGrid(thumbnailStore, length, spacing, pixelIds, colCount,
                       bg=(255, 255, 255), leftLabel=None,
                       textColour=(0, 0, 0), fontsize=None, topLabel=None):
    """
    Retrieves thumbnails for each pixelId, and places them in a grid,
    with White background.
    Option to add a vertical label to the left of the canvas
    Creates a PIL 'Image' which is returned

    :param thumbnailStore:  The omero thumbnail store.
    :param length:			Length of longest thumbnail side. int
    :param spacing:			The spacing between thumbnails and around the
                            edges. int
    :param pixelIds:		List of pixel IDs. [long]
    :param colCount:		The number of columns. int
    :param bg:				Background colour as (r,g,b).
                            Default is white (255,255,255)
    :param leftLabel: 		Optional string to display vertically to the left.
    :param textColour:		The colour of the text as (r,g,b).
                            Default is black (0,0,0)
    :param fontsize:		Size of the font.
                            Default is calculated based on thumbnail length.
                            int
    :return: 			    The PIL Image canvas.
    """
    warnings.warn(
        "This module is deprecated as of OMERO 5.3.0", DeprecationWarning)
    mode = "RGB"
    # work out how many rows and columns are needed for all the images
    imgCount = len(pixelIds)

    rowCount = (old_div(imgCount, colCount))
    # check that we have enough rows and cols...
    while (colCount * rowCount) < imgCount:
        rowCount += 1

    leftSpace = topSpace = spacing
    minWidth = 0

    textHeight = 0
    if leftLabel or topLabel:
        # if no images (no rows), need to make at least one row to show label
        if leftLabel is not None and rowCount == 0:
            rowCount = 1
        if fontsize is None:
            fontsize = old_div(length, 10) + 5
        font = getFont(fontsize)
        if leftLabel:
            textWidth, textHeight = font.getsize(leftLabel)
            leftSpace = spacing + textHeight + spacing
        if topLabel:
            textWidth, textHeight = font.getsize(topLabel)
            topSpace = spacing + textHeight + spacing
            minWidth = leftSpace + textWidth + spacing

    # work out the canvas size needed, and create a white canvas
    colsNeeded = min(colCount, imgCount)
    canvasWidth = max(minWidth, (leftSpace + colsNeeded * (length + spacing)))
    canvasHeight = topSpace + rowCount * (length + spacing) + spacing
    mode = "RGB"
    size = (canvasWidth, canvasHeight)
    canvas = Image.new(mode, size, bg)

    # to write text up the left side, need to write it on horizontal canvas
    # and rotate.
    if leftLabel:
        labelCanvasWidth = canvasHeight
        labelCanvasHeight = textHeight + spacing
        labelSize = (labelCanvasWidth, labelCanvasHeight)
        textCanvas = Image.new(mode, labelSize, bg)
        draw = ImageDraw.Draw(textCanvas)
        textWidth = font.getsize(leftLabel)[0]
        textX = old_div((labelCanvasWidth - textWidth), 2)
        draw.text((textX, spacing), leftLabel, font=font, fill=textColour)
        verticalCanvas = textCanvas.rotate(90)
        pasteImage(verticalCanvas, canvas, 0, 0)
        del draw

    if topLabel is not None:
        labelCanvasWidth = canvasWidth
        labelCanvasHeight = textHeight + spacing
        labelSize = (labelCanvasWidth, labelCanvasHeight)
        textCanvas = Image.new(mode, labelSize, bg)
        draw = ImageDraw.Draw(textCanvas)
        draw.text((spacing, spacing), topLabel, font=font, fill=textColour)
        pasteImage(textCanvas, canvas, leftSpace, 0)
        del draw

    # loop through the images, getting a thumbnail and placing it on a new row
    # and column
    r = 0
    c = 0
    thumbnailMap = getThumbnailSet(thumbnailStore, length, pixelIds)
    for pixelsId in pixelIds:
        if pixelsId in thumbnailMap:
            # getThumbnail(thumbnailStore, pixelsId, length)
            thumbnail = thumbnailMap[pixelsId]
            # check we have a thumbnail (won't get one if image is invalid)
            if thumbnail:
                # make an "Image" from the string-encoded thumbnail
                thumbImage = Image.open(io.StringIO(thumbnail))
                # paste the image onto the canvas at the correct coordinates
                # for the current row and column
                x = c * (length + spacing) + leftSpace
                y = r * (length + spacing) + topSpace
                pasteImage(thumbImage, canvas, x, y)

        # increment the column, and if we're at the last column, start a new
        # row
        c = c + 1
        if c == colCount:
            c = 0
            r = r + 1

    return canvas


def checkRGBRange(value):
    """
    Checks that the value is between 0 and 255. Returns integer value
    If the value is not valid, return 255
    (better to see something than nothing!)

    :param value:		The value to check.
    :return:			An integer between 0 and 255
    """
    warnings.warn(
        "This module is deprecated as of OMERO 5.3.0", DeprecationWarning)
    try:
        v = int(value)
        if 0 <= v <= 255:
            return v
    except:
        return 255


def RGBIntToRGBA(RGB):
    """
    Returns a tuple of (r,g,b,a) from an integer colour
    r, g, b, a are 0-255.

    :param RGB:		A colour as integer. Int
    :return:		A tuple of (r,g,b,a)
    """
    warnings.warn(
        "This module is deprecated as of OMERO 5.3.0", DeprecationWarning)
    r = checkRGBRange((RGB >> 16) & 0xFF)
    g = checkRGBRange((RGB >> 8) & 0xFF)
    b = checkRGBRange((RGB >> 0) & 0xFF)
    a = checkRGBRange((RGB >> 24) & 0xFF)
    if a == 0:
        a = 255
    return (r, g, b, a)


def RGBIntToRGB(RGB):
    """
    Returns a tuple of (r,g,b) from an integer colour
    r, g, b are 0-255.

    :param RGB:		A colour as integer. Int
    :return:		A tuple of (r,g,b)
    """
    warnings.warn(
        "This module is deprecated as of OMERO 5.3.0", DeprecationWarning)
    r, g, b, a = RGBIntToRGBA(RGB)
    return (r, g, b)


def getZoomFactor(imageSize, maxW, maxH):
    """
    Returns the factor by which the Image has to be shrunk
    so that its dimensions are less that maxW and maxH
    E.g. If the image must be half-sized, this method returns 2.0 (float)

    :param imageSize: 		Size of the image as tuple (width, height)
    :param maxW:			The max width after zooming
    :param maxH:			The max height after zooming
    :return:			    The factor by which to shrink the image to be
                            within max width and height
    """
    warnings.warn(
        "This module is deprecated as of OMERO 5.3.0", DeprecationWarning)
    imageW, imageH = imageSize
    zoomW = old_div(float(imageW), float(maxW))
    zoomH = old_div(float(imageH), float(maxH))
    return max(zoomW, zoomH)


def resizeImage(image, maxW, maxH):
    """
    Resize the image so that it is as big as possible,
    within the dimensions maxW, maxH

    :param image:		The PIL Image to zoom
    :param maxW:		The max width of the zoomed image
    :param maxH:		The max height of the zoomed image
    :return:		The zoomed image. PIL Image.
    """
    warnings.warn(
        "This module is deprecated as of OMERO 5.3.0", DeprecationWarning)
    imageW, imageH = image.size
    if imageW == maxW and imageH == maxH:
        return image
    # find which axis requires the biggest zoom (smallest relative max
    # dimension)
    zoomW = old_div(float(imageW), float(maxW))
    zoomH = old_div(float(imageH), float(maxH))
    zoom = max(zoomW, zoomH)
    if zoomW >= zoomH:  # size is defined by width
        maxH = int(imageH // zoom)  # calculate the new height
    else:
        maxW = int(imageW // zoom)
    return image.resize((maxW, maxH))
